SystemOrganization addCategory: #'PocketCube-Tests'!
SystemOrganization addCategory: #'PocketCube-Morph'!
SystemOrganization addCategory: #PocketCube!
SystemOrganization addCategory: #'PocketCube-Recognizer'!
SystemOrganization addCategory: #'PocketCube-SidePatterns'!
SystemOrganization addCategory: #'PocketCube-SidePatterns-Tests'!

!ReadStream methodsFor: '*PocketCube' stamp: 'hjo 3/25/2011 10:56'!
nextInt: anInteger
	"Answer a random integer in the interval [1, anInteger].
	Handle large numbers too (for cryptography)."

	anInteger strictlyPositive ifFalse: [ self error: 'Range must be positive' ].
	anInteger asFloat isInfinite
		ifTrue: [^(self next asFraction * anInteger) truncated + 1].
	^ (self next * anInteger) truncated + 1! !

RectangleMorph subclass: #PocketCubeGameMorph
	instanceVariableNames: 'controller'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-Morph'!

!PocketCubeGameMorph methodsFor: 'as yet unclassified' stamp: 'hjo 7/31/2010 19:42'!
controller

	^controller! !

!PocketCubeGameMorph methodsFor: 'as yet unclassified' stamp: 'hjo 7/31/2010 19:42'!
controller: aPocketCubeController 

	controller := aPocketCubeController 
! !

!PocketCubeGameMorph methodsFor: 'as yet unclassified' stamp: 'hjo 7/31/2010 19:23'!
handlesKeyboard: evt

	^true! !

!PocketCubeGameMorph methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 20:46'!
initialize 

	super initialize.
	self openInWorld.
	self bounds: (0@0 corner: 640@480).
	self extent:  454@454.
	self position: 0@0.
	self color: Color white.
! !

!PocketCubeGameMorph methodsFor: 'as yet unclassified' stamp: 'hjo 7/31/2010 19:49'!
keyStroke: anEvent

	anEvent keyValue = 97 ifTrue: [^self controller rotateLeft].
	anEvent keyValue = 115 ifTrue: [^self controller rotateRight].
	anEvent keyValue = 122 ifTrue: [^self controller rotateClockwise].
	anEvent keyValue = 120 ifTrue: [^self controller rotateCounterClockwise].	
	anEvent keyValue = 106 ifTrue: [^self controller twistUpperLayerLeft].	
	anEvent keyValue = 107 ifTrue: [^self controller twistUpperLayerRight].
	anEvent keyValue = 110 ifTrue: [^self controller twistLowerLayerLeft].
	anEvent keyValue = 109 ifTrue: [^self controller twistLowerLayerRight].
	Transcript show: anEvent keyValue; cr! !

TestCase subclass: #PocketCubeColorPieceTest
	instanceVariableNames: 'pocketCube hasReceivedHasPocketCubeChanged'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-Tests'!

!PocketCubeColorPieceTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/2/2011 21:08'!
onPocketCubeChanged

	hasReceivedHasPocketCubeChanged := true.! !

!PocketCubeColorPieceTest methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 15:51'!
setUp

	super setUp.
	hasReceivedHasPocketCubeChanged := false.
	pocketCube := PocketCubeWithMemory new.
	pocketCube initializeSidesWithColors.
	pocketCube when: #pocketCubeChanged send: #onPocketCubeChanged  to: self! !

!PocketCubeColorPieceTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/2/2011 19:11'!
testSidesAreUpperLayerTwistedLeft

	self deny: pocketCube sidesAreUpperLayerTwistedLeft.
	pocketCube twistUpperLayerLeft.
	self assert: pocketCube sidesAreUpperLayerTwistedLeft.! !

!PocketCubeColorPieceTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/2/2011 19:11'!
testSidesAreUpperLayerTwistedRight

	self deny: pocketCube sidesAreUpperLayerTwistedRight.
	pocketCube twistUpperLayerRight.
	self assert: pocketCube sidesAreUpperLayerTwistedRight.! !

!PocketCubeColorPieceTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/2/2011 19:13'!
testSidesAreUpperLayerTwistedTwice

	self deny: pocketCube sidesAreUpperLayerTwistedTwice.
	2 timesRepeat: [pocketCube twistUpperLayerRight].
	self assert: pocketCube sidesAreUpperLayerTwistedTwice.! !

!PocketCubeColorPieceTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/2/2011 21:10'!
testTwistLowerLayerLeft

	pocketCube twistLowerLayerLeft.
	self assert: hasReceivedHasPocketCubeChanged ! !

!PocketCubeColorPieceTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/2/2011 21:09'!
testTwistLowerLayerRight

	pocketCube twistLowerLayerRight.
	self assert: hasReceivedHasPocketCubeChanged ! !

!PocketCubeColorPieceTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/2/2011 21:11'!
testTwistUpperLayerLeft

	pocketCube twistUpperLayerLeft.
	self assert: hasReceivedHasPocketCubeChanged ! !

!PocketCubeColorPieceTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/2/2011 21:11'!
testTwistUpperLayerRight

	pocketCube twistUpperLayerRight.
	self assert: hasReceivedHasPocketCubeChanged ! !

TestCase subclass: #PocketCubeNumberPieceTest
	instanceVariableNames: 'pocketCube'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-Tests'!

!PocketCubeNumberPieceTest methodsFor: 'running' stamp: 'hjo 3/24/2011 15:52'!
setUp

	super setUp.
	pocketCube := PocketCubeWithMemory new.
	pocketCube topSide: (PocketCubeSide upperLeftPiece: 1 upperRightPiece: 2 lowerLeftPiece: 3 lowerRightPiece: 4).
	pocketCube leftSide: (PocketCubeSide upperLeftPiece: 5 upperRightPiece: 6 lowerLeftPiece: 13 lowerRightPiece: 14).
	pocketCube bottomSide: (PocketCubeSide upperLeftPiece: 21 upperRightPiece: 22 lowerLeftPiece: 23 lowerRightPiece: 24).
	pocketCube rightSide: (PocketCubeSide upperLeftPiece: 9 upperRightPiece: 10 lowerLeftPiece: 17 lowerRightPiece: 18).
	pocketCube frontSide: (PocketCubeSide upperLeftPiece: 7 upperRightPiece: 8 lowerLeftPiece: 15 lowerRightPiece: 16).
	pocketCube backSide: (PocketCubeSide upperLeftPiece: 11 upperRightPiece: 12 lowerLeftPiece: 19 lowerRightPiece: 20).! !

!PocketCubeNumberPieceTest methodsFor: 'setup testing' stamp: 'hjo 7/18/2010 07:36'!
testBackSide

	self assert: pocketCube backSide = (PocketCubeSide upperLeftPiece:11 upperRightPiece: 12 lowerLeftPiece: 19 lowerRightPiece: 20)! !

!PocketCubeNumberPieceTest methodsFor: 'setup testing' stamp: 'hjo 7/18/2010 07:36'!
testBottomSide

	self assert: pocketCube bottomSide = (PocketCubeSide upperLeftPiece: 21 upperRightPiece: 22 lowerLeftPiece: 23 lowerRightPiece: 24)! !

!PocketCubeNumberPieceTest methodsFor: 'setup testing' stamp: 'hjo 7/18/2010 07:36'!
testFrontSide

	self assert: pocketCube frontSide = (PocketCubeSide upperLeftPiece: 7 upperRightPiece: 8 lowerLeftPiece: 15 lowerRightPiece: 16)! !

!PocketCubeNumberPieceTest methodsFor: 'setup testing' stamp: 'hjo 7/18/2010 07:37'!
testLeftSide

	self assert: pocketCube leftSide = (PocketCubeSide upperLeftPiece: 5 upperRightPiece: 6 lowerLeftPiece: 13 lowerRightPiece: 14)! !

!PocketCubeNumberPieceTest methodsFor: 'setup testing' stamp: 'hjo 7/18/2010 07:37'!
testRightSide

	self assert: pocketCube rightSide = (PocketCubeSide upperLeftPiece: 9 upperRightPiece: 10 lowerLeftPiece: 17 lowerRightPiece: 18)! !

!PocketCubeNumberPieceTest methodsFor: 'setup testing' stamp: 'hjo 2/7/2011 10:21'!
testSideColumn

	|sides|
	sides := pocketCube sideColumn.
	self assert: (PocketCubeSide upperLeftPiece: 2 upperRightPiece: 4 lowerLeftPiece: 1 lowerRightPiece: 3) equals: (sides at: 1).
	self assert: (PocketCubeSide upperLeftPiece: 8 upperRightPiece: 16 lowerLeftPiece: 7 lowerRightPiece: 15) equals: (sides at: 2).
	self assert: (PocketCubeSide upperLeftPiece: 22 upperRightPiece: 24 lowerLeftPiece: 21 lowerRightPiece: 23) equals: (sides at: 3).
	self assert: (PocketCubeSide upperLeftPiece: 19 upperRightPiece: 11 lowerLeftPiece: 20 lowerRightPiece: 12) equals: (sides at: 4).! !

!PocketCubeNumberPieceTest methodsFor: 'setup testing' stamp: 'hjo 2/6/2011 19:25'!
testSideRow

	|sides|
	sides := pocketCube sideRow.
	self assert: (PocketCubeSide upperLeftPiece: 5 upperRightPiece: 6 lowerLeftPiece: 13 lowerRightPiece: 14) equals: (sides at: 1).
	self assert: (PocketCubeSide upperLeftPiece: 7 upperRightPiece: 8 lowerLeftPiece: 15 lowerRightPiece: 16) equals: (sides at: 2).
	self assert: (PocketCubeSide upperLeftPiece: 9 upperRightPiece: 10 lowerLeftPiece: 17 lowerRightPiece: 18) equals: (sides at: 3).
	self assert: (PocketCubeSide upperLeftPiece: 11 upperRightPiece: 12 lowerLeftPiece: 19 lowerRightPiece: 20) equals: (sides at: 4).! !

!PocketCubeNumberPieceTest methodsFor: 'setup testing' stamp: 'hjo 7/18/2010 07:37'!
testTopSide

	self assert: pocketCube topSide = (PocketCubeSide upperLeftPiece: 1 upperRightPiece: 2 lowerLeftPiece: 3 lowerRightPiece: 4)! !

PocketCubeNumberPieceTest subclass: #PocketCubeRotateClockwiseTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-Tests'!

!PocketCubeRotateClockwiseTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 09:52'!
setUp

	super setUp.
	pocketCube rotateClockwise.! !

!PocketCubeRotateClockwiseTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 09:53'!
testBackSide

	self assert: pocketCube backSide = (PocketCubeSide upperLeftPiece:12 upperRightPiece: 20 lowerLeftPiece: 11 lowerRightPiece: 19)! !

!PocketCubeRotateClockwiseTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 09:55'!
testBottomSide

	self assert: pocketCube bottomSide = (PocketCubeSide upperLeftPiece: 17 upperRightPiece: 9 lowerLeftPiece: 18 lowerRightPiece: 10)! !

!PocketCubeRotateClockwiseTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 09:54'!
testFrontSide

	self assert: pocketCube frontSide = (PocketCubeSide upperLeftPiece:15 upperRightPiece: 7 lowerLeftPiece: 16 lowerRightPiece: 8)! !

!PocketCubeRotateClockwiseTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 09:55'!
testLeftSide

	self assert: pocketCube leftSide = (PocketCubeSide upperLeftPiece: 23 upperRightPiece: 21 lowerLeftPiece: 24 lowerRightPiece: 22)! !

!PocketCubeRotateClockwiseTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 09:56'!
testRightSide

	self assert: pocketCube rightSide = (PocketCubeSide upperLeftPiece: 3 upperRightPiece: 1 lowerLeftPiece: 4 lowerRightPiece: 2)! !

!PocketCubeRotateClockwiseTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 09:56'!
testTopSide

	self assert: pocketCube topSide = (PocketCubeSide upperLeftPiece: 13 upperRightPiece: 5 lowerLeftPiece: 14 lowerRightPiece: 6)! !

PocketCubeNumberPieceTest subclass: #PocketCubeRotateCounterClockwiseTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-Tests'!

!PocketCubeRotateCounterClockwiseTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 22:13'!
setUp

	super setUp.
	pocketCube rotateCounterClockwise.! !

!PocketCubeRotateCounterClockwiseTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 22:15'!
testBackSide

	self assert: pocketCube backSide = (PocketCubeSide upperLeftPiece:19 upperRightPiece: 11 lowerLeftPiece: 20 lowerRightPiece: 12)! !

!PocketCubeRotateCounterClockwiseTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 22:16'!
testBottomSide

	self assert: pocketCube bottomSide = (PocketCubeSide upperLeftPiece: 6 upperRightPiece: 14 lowerLeftPiece: 5 lowerRightPiece: 13)! !

!PocketCubeRotateCounterClockwiseTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 22:16'!
testFrontSide

	self assert: pocketCube frontSide = (PocketCubeSide upperLeftPiece:8 upperRightPiece: 16 lowerLeftPiece: 7 lowerRightPiece: 15)! !

!PocketCubeRotateCounterClockwiseTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 22:17'!
testLeftSide

	self assert: pocketCube leftSide = (PocketCubeSide upperLeftPiece: 2 upperRightPiece: 4 lowerLeftPiece: 1 lowerRightPiece: 3)! !

!PocketCubeRotateCounterClockwiseTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 22:17'!
testRightSide

	self assert: pocketCube rightSide = (PocketCubeSide upperLeftPiece: 22 upperRightPiece: 24 lowerLeftPiece: 21 lowerRightPiece: 23)! !

!PocketCubeRotateCounterClockwiseTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 22:18'!
testTopSide

	self assert: pocketCube topSide = (PocketCubeSide upperLeftPiece: 10 upperRightPiece: 18 lowerLeftPiece: 9 lowerRightPiece: 17)! !

PocketCubeNumberPieceTest subclass: #PocketCubeRotateLeftTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-Tests'!

!PocketCubeRotateLeftTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 01:35'!
setUp

	super setUp.
	pocketCube rotateLeft.! !

!PocketCubeRotateLeftTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:43'!
testBackSide

	self assert: pocketCube backSide = (PocketCubeSide upperLeftPiece:9 upperRightPiece: 10 lowerLeftPiece: 17 lowerRightPiece: 18)! !

!PocketCubeRotateLeftTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 09:23'!
testBottomSide

	self assert: pocketCube bottomSide = (PocketCubeSide upperLeftPiece: 23 upperRightPiece: 21 lowerLeftPiece: 24 lowerRightPiece: 22)! !

!PocketCubeRotateLeftTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 09:38'!
testFrontSide

	self assert: pocketCube frontSide = (PocketCubeSide upperLeftPiece:5 upperRightPiece: 6 lowerLeftPiece: 13 lowerRightPiece: 14)! !

!PocketCubeRotateLeftTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 09:30'!
testLeftSide

	self assert: pocketCube leftSide = (PocketCubeSide upperLeftPiece: 11 upperRightPiece: 12 lowerLeftPiece: 19 lowerRightPiece: 20)! !

!PocketCubeRotateLeftTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 09:33'!
testRightSide

	self assert: pocketCube rightSide = (PocketCubeSide upperLeftPiece: 7 upperRightPiece: 8 lowerLeftPiece: 15 lowerRightPiece: 16)! !

!PocketCubeRotateLeftTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:40'!
testTopSide

	self assert: pocketCube topSide = (PocketCubeSide upperLeftPiece: 2 upperRightPiece: 4 lowerLeftPiece: 1 lowerRightPiece: 3)! !

PocketCubeNumberPieceTest subclass: #PocketCubeRotateRightTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-Tests'!

!PocketCubeRotateRightTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 09:29'!
setUp

	super setUp.
	pocketCube rotateRight.! !

!PocketCubeRotateRightTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 09:34'!
testBackSide

	self assert: pocketCube backSide = (PocketCubeSide upperLeftPiece:5 upperRightPiece: 6 lowerLeftPiece: 13 lowerRightPiece: 14)! !

!PocketCubeRotateRightTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 09:34'!
testBottomSide

	self assert: pocketCube bottomSide = (PocketCubeSide upperLeftPiece: 22 upperRightPiece: 24 lowerLeftPiece: 21 lowerRightPiece: 23)! !

!PocketCubeRotateRightTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 09:37'!
testFrontSide

	self assert: pocketCube frontSide = (PocketCubeSide upperLeftPiece:9 upperRightPiece: 10 lowerLeftPiece: 17 lowerRightPiece: 18)! !

!PocketCubeRotateRightTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 09:39'!
testLeftSide

	self assert: pocketCube leftSide = (PocketCubeSide upperLeftPiece: 7 upperRightPiece: 8 lowerLeftPiece: 15 lowerRightPiece: 16)! !

!PocketCubeRotateRightTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 09:39'!
testRightSide

	self assert: pocketCube rightSide = (PocketCubeSide upperLeftPiece: 11 upperRightPiece: 12 lowerLeftPiece: 19 lowerRightPiece: 20)! !

!PocketCubeRotateRightTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 09:37'!
testTopSide

	self assert: pocketCube topSide = (PocketCubeSide upperLeftPiece: 3 upperRightPiece: 1 lowerLeftPiece: 4 lowerRightPiece: 2)! !

PocketCubeNumberPieceTest subclass: #PocketCubeWhenTwistLowerLayerLeft
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-Tests'!

!PocketCubeWhenTwistLowerLayerLeft methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:15'!
setUp

	super setUp.
	pocketCube twistLowerLayerLeft.! !

!PocketCubeWhenTwistLowerLayerLeft methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:19'!
testBackSide

	self assert: pocketCube backSide = (PocketCubeSide upperLeftPiece: 11 upperRightPiece: 12 lowerLeftPiece: 13 lowerRightPiece: 14).! !

!PocketCubeWhenTwistLowerLayerLeft methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:20'!
testBottomSide

	self assert: pocketCube bottomSide = (PocketCubeSide upperLeftPiece: 22 upperRightPiece: 24 lowerLeftPiece: 21 lowerRightPiece: 23)! !

!PocketCubeWhenTwistLowerLayerLeft methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:18'!
testFrontSide

	self assert: pocketCube frontSide = (PocketCubeSide upperLeftPiece: 7 upperRightPiece: 8 lowerLeftPiece: 17 lowerRightPiece: 18).! !

!PocketCubeWhenTwistLowerLayerLeft methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:18'!
testLeftSide

	self assert: pocketCube leftSide = (PocketCubeSide upperLeftPiece: 5 upperRightPiece: 6  lowerLeftPiece: 15 lowerRightPiece: 16)! !

!PocketCubeWhenTwistLowerLayerLeft methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:19'!
testRightSide

	self assert: pocketCube rightSide = (PocketCubeSide upperLeftPiece: 9 upperRightPiece: 10 lowerLeftPiece: 19 lowerRightPiece: 20)! !

!PocketCubeWhenTwistLowerLayerLeft methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:20'!
testTopSide

	self assert: pocketCube topSide = (PocketCubeSide upperLeftPiece: 1 upperRightPiece: 2  lowerLeftPiece: 3 lowerRightPiece: 4)! !

PocketCubeNumberPieceTest subclass: #PocketCubeWhenTwistLowerLayerRight
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-Tests'!

!PocketCubeWhenTwistLowerLayerRight methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:22'!
setUp

	super setUp.
	pocketCube twistLowerLayerRight! !

!PocketCubeWhenTwistLowerLayerRight methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:25'!
testBackSide

	self assert: pocketCube backSide = (PocketCubeSide upperLeftPiece: 11 upperRightPiece: 12 lowerLeftPiece: 17 lowerRightPiece: 18)! !

!PocketCubeWhenTwistLowerLayerRight methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:23'!
testBottomSide

	self assert: pocketCube bottomSide = (PocketCubeSide upperLeftPiece: 23 upperRightPiece: 21 lowerLeftPiece: 24 lowerRightPiece: 22).! !

!PocketCubeWhenTwistLowerLayerRight methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:24'!
testFrontSide

	self assert: pocketCube frontSide = (PocketCubeSide upperLeftPiece: 7 upperRightPiece: 8 lowerLeftPiece: 13 lowerRightPiece: 14)! !

!PocketCubeWhenTwistLowerLayerRight methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:23'!
testLeftSide

	self assert: pocketCube leftSide = (PocketCubeSide upperLeftPiece: 5 upperRightPiece: 6  lowerLeftPiece: 19 lowerRightPiece: 20)! !

!PocketCubeWhenTwistLowerLayerRight methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:24'!
testRightSide

	self assert: pocketCube rightSide = (PocketCubeSide upperLeftPiece: 9 upperRightPiece: 10 lowerLeftPiece: 15 lowerRightPiece: 16)! !

!PocketCubeWhenTwistLowerLayerRight methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:22'!
testTopSide

	self assert: pocketCube topSide = (PocketCubeSide upperLeftPiece: 1 upperRightPiece: 2  lowerLeftPiece: 3 lowerRightPiece: 4)! !

PocketCubeNumberPieceTest subclass: #PocketCubeWhenTwistUpperLayerLeft
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-Tests'!

!PocketCubeWhenTwistUpperLayerLeft methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:27'!
setUp

	super setUp.
	pocketCube twistUpperLayerLeft! !

!PocketCubeWhenTwistUpperLayerLeft methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:28'!
testBackSide

	self assert: pocketCube backSide = (PocketCubeSide upperLeftPiece: 5 upperRightPiece: 6 lowerLeftPiece: 19 lowerRightPiece: 20)! !

!PocketCubeWhenTwistUpperLayerLeft methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:28'!
testBottomSide

	self assert: pocketCube bottomSide = (PocketCubeSide upperLeftPiece: 21 upperRightPiece: 22 lowerLeftPiece: 23 lowerRightPiece: 24).! !

!PocketCubeWhenTwistUpperLayerLeft methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:27'!
testFrontSide

	self assert: pocketCube frontSide = (PocketCubeSide upperLeftPiece: 9 upperRightPiece: 10 lowerLeftPiece: 15 lowerRightPiece: 16)! !

!PocketCubeWhenTwistUpperLayerLeft methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:27'!
testLeftSide

	self assert: pocketCube leftSide = (PocketCubeSide upperLeftPiece: 7 upperRightPiece: 8  lowerLeftPiece: 13 lowerRightPiece: 14)! !

!PocketCubeWhenTwistUpperLayerLeft methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:28'!
testRightSide

	self assert: pocketCube rightSide = (PocketCubeSide upperLeftPiece: 11 upperRightPiece: 12 lowerLeftPiece: 17 lowerRightPiece: 18)! !

!PocketCubeWhenTwistUpperLayerLeft methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:29'!
testTopSide

	self assert: pocketCube topSide = (PocketCubeSide upperLeftPiece: 3 upperRightPiece: 1 lowerLeftPiece: 4 lowerRightPiece: 2)! !

PocketCubeNumberPieceTest subclass: #PocketCubeWhenTwistUpperLayerRight
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-Tests'!

!PocketCubeWhenTwistUpperLayerRight methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:30'!
setUp

	super setUp.
	pocketCube twistUpperLayerRight! !

!PocketCubeWhenTwistUpperLayerRight methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:31'!
testBackSide

	self assert: pocketCube backSide = (PocketCubeSide upperLeftPiece: 9 upperRightPiece: 10 lowerLeftPiece: 19 lowerRightPiece: 20)! !

!PocketCubeWhenTwistUpperLayerRight methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:30'!
testBottomSide

	self assert: pocketCube bottomSide = (PocketCubeSide upperLeftPiece: 21 upperRightPiece: 22 lowerLeftPiece: 23 lowerRightPiece: 24).! !

!PocketCubeWhenTwistUpperLayerRight methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:31'!
testFrontSide

	self assert: pocketCube frontSide = (PocketCubeSide upperLeftPiece: 5 upperRightPiece: 6 lowerLeftPiece: 15 lowerRightPiece: 16)! !

!PocketCubeWhenTwistUpperLayerRight methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:31'!
testLeftSide

	self assert: pocketCube leftSide = (PocketCubeSide upperLeftPiece: 11 upperRightPiece: 12  lowerLeftPiece: 13 lowerRightPiece: 14)! !

!PocketCubeWhenTwistUpperLayerRight methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:31'!
testRightSide

	self assert: pocketCube rightSide = (PocketCubeSide upperLeftPiece: 7 upperRightPiece: 8 lowerLeftPiece: 17 lowerRightPiece: 18)! !

!PocketCubeWhenTwistUpperLayerRight methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:32'!
testTopSide

	self assert: pocketCube topSide = (PocketCubeSide upperLeftPiece: 2 upperRightPiece: 4 lowerLeftPiece: 1 lowerRightPiece: 3)! !

TestCase subclass: #PocketCubePatternGeneratorTest
	instanceVariableNames: 'pocketcube patternGenerator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-Tests'!

!PocketCubePatternGeneratorTest methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 15:18'!
setUp

	super setUp.
	pocketcube := PocketCubeWithMemory new.
	pocketcube initializeSidesWithColors.
	patternGenerator := PocketCubePatternGenerator on: pocketcube withScrambler: pocketcube
! !

!PocketCubePatternGeneratorTest methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 15:31'!
testSolved

	self assert: 'isPatternOf: aPocketCube

^(Set with: aPocketCube rightSide upperLeftPiece with: aPocketCube rightSide upperRightPiece with: aPocketCube rightSide lowerLeftPiece with: aPocketCube rightSide lowerRightPiece) size = 1 and: [
(Set with: aPocketCube backSide upperLeftPiece with: aPocketCube backSide upperRightPiece with: aPocketCube backSide lowerLeftPiece with: aPocketCube backSide lowerRightPiece) size = 1 and: [
(Set with: aPocketCube frontSide upperLeftPiece with: aPocketCube frontSide upperRightPiece with: aPocketCube frontSide lowerLeftPiece with: aPocketCube frontSide lowerRightPiece) size = 1 and: [
(Set with: aPocketCube topSide upperLeftPiece with: aPocketCube topSide upperRightPiece with: aPocketCube topSide lowerLeftPiece with: aPocketCube topSide lowerRightPiece) size = 1 and: [
(Set with: aPocketCube leftSide upperLeftPiece with: aPocketCube leftSide upperRightPiece with: aPocketCube leftSide lowerLeftPiece with: aPocketCube leftSide lowerRightPiece) size = 1 and: [
(Set with: aPocketCube bottomSide upperLeftPiece with: aPocketCube bottomSide upperRightPiece with: aPocketCube bottomSide lowerLeftPiece with: aPocketCube bottomSide lowerRightPiece) size = 1]]]]]'
	equals: patternGenerator implementationStringOfIsPatternOf.
	
	self assert: 'oneStepBetterFor: aPocketCube 

	"do nothing"' equals: patternGenerator implementationStringOfOneStepBetterDoNothing! !

!PocketCubePatternGeneratorTest methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 15:34'!
testTwistUpperLayerLeft

	pocketcube twistUpperLayerLeft.
	self assert: 'isPatternOf: aPocketCube

^(Set with: aPocketCube frontSide upperLeftPiece with: aPocketCube frontSide upperRightPiece with: aPocketCube rightSide lowerLeftPiece with: aPocketCube rightSide lowerRightPiece) size = 1 and: [
(Set with: aPocketCube rightSide upperLeftPiece with: aPocketCube rightSide upperRightPiece with: aPocketCube backSide lowerLeftPiece with: aPocketCube backSide lowerRightPiece) size = 1 and: [
(Set with: aPocketCube frontSide lowerLeftPiece with: aPocketCube frontSide lowerRightPiece with: aPocketCube leftSide upperLeftPiece with: aPocketCube leftSide upperRightPiece) size = 1 and: [
(Set with: aPocketCube topSide upperLeftPiece with: aPocketCube topSide upperRightPiece with: aPocketCube topSide lowerLeftPiece with: aPocketCube topSide lowerRightPiece) size = 1 and: [
(Set with: aPocketCube leftSide lowerLeftPiece with: aPocketCube leftSide lowerRightPiece with: aPocketCube backSide upperLeftPiece with: aPocketCube backSide upperRightPiece) size = 1 and: [
(Set with: aPocketCube bottomSide upperLeftPiece with: aPocketCube bottomSide upperRightPiece with: aPocketCube bottomSide lowerLeftPiece with: aPocketCube bottomSide lowerRightPiece) size = 1]]]]]'
	equals: patternGenerator implementationStringOfIsPatternOf.
	
	self assert: 'oneStepBetterFor: aPocketCube 

	aPocketCube twistUpperLayerRight.
	aPocketCube forgetLastOperation.
	aPocketCube forgetLastOperation.' equals: patternGenerator implementationStringOfOneStepBetterWithAction! !

TestCase subclass: #PocketCubeScrambleOperationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-Tests'!

!PocketCubeScrambleOperationTest methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 10:02'!
testX1

	self assert: {PoTwistLowerLayerLeft singleton} equals: (ScrambleX new amount: 1) operations
	! !

!PocketCubeScrambleOperationTest methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 10:02'!
testX2

	self assert: {PoTwistLowerLayerLeft singleton. PoTwistLowerLayerLeft singleton} equals: (ScrambleX new amount: 2) operations! !

!PocketCubeScrambleOperationTest methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 10:02'!
testY1

	self assert: ({PoRotateCounterClockwise. PoTwistUpperLayerRight. PoRotateClockwise} collect: [:each| each singleton]) equals: (ScrambleY new amount: 1) operations
	! !

!PocketCubeScrambleOperationTest methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 10:03'!
testY2

	self assert: ({PoRotateCounterClockwise. PoTwistUpperLayerRight. PoTwistUpperLayerRight. PoRotateClockwise} collect: [:each| each singleton]) equals:  (ScrambleY new amount: 2) operations
	! !

!PocketCubeScrambleOperationTest methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 10:03'!
testYNeg1

	self assert: ({PoRotateCounterClockwise. PoTwistUpperLayerLeft. PoRotateClockwise} collect: [:each| each  singleton]) equals: (ScrambleY new amount: -1) operations! !

!PocketCubeScrambleOperationTest methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 10:03'!
testZ1

	self assert: ({PoRotateRight. PoRotateCounterClockwise. PoTwistLowerLayerLeft. PoRotateClockwise . PoRotateLeft} collect: [:each| each singleton]) equals: (ScrambleZ new amount: 1) operations
	! !

!PocketCubeScrambleOperationTest methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 10:03'!
testZ2

	self assert: ({PoRotateRight. PoRotateCounterClockwise. PoTwistLowerLayerLeft. PoTwistLowerLayerLeft . PoRotateClockwise. PoRotateLeft} collect: [:each| each singleton]) equals: (ScrambleZ new amount: 2) operations
	! !

!PocketCubeScrambleOperationTest methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 10:04'!
testZNeg1

	self assert: ({PoRotateRight. PoRotateCounterClockwise. PoTwistLowerLayerRight. PoRotateClockwise. PoRotateLeft} collect: [:each| each singleton]) equals: (ScrambleZ new amount: -1) operations! !

TestCase subclass: #PocketCubeScramblerTest
	instanceVariableNames: 'pocketcube scrambler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-Tests'!

!PocketCubeScramblerTest methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 12:25'!
setUp

	super setUp.
	pocketcube := PocketCube new.
	pocketcube initializeSidesWithColors.
	scrambler := PocketCubeScrambler new.
	scrambler scrambleSequenceOf: 5 randomStream: 
	{0.98378020.  0.1111.   0.23984.   0.87531112.
	0.49912. 0.5939210. 0.001123. 0.777777. 0.399483.
	0.50003.  0.839120.  0.425095.  0.6930294. 0.10101010
	} readStream! !

!PocketCubeScramblerTest methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 12:37'!
testInverseOperationSelector

	self assert: PoRotateCounterClockwise singleton selector equals: scrambler firstInverseOperation selector
	
! !

!PocketCubeScramblerTest methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 10:35'!
testScrambleUnscramble

	self assert: pocketcube allSidesAreOneColor.
	scrambler applyTo: pocketcube.
	self deny: pocketcube allSidesAreOneColor.
	scrambler rollbackTo: pocketcube.
	self assert: pocketcube allSidesAreOneColor.! !

TestCase subclass: #PocketCubeSideTest
	instanceVariableNames: 'side'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-Tests'!

!PocketCubeSideTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/17/2010 19:45'!
setUp

	super setUp.
	side := PocketCubeSide upperLeftPiece: 1 upperRightPiece: 2 lowerLeftPiece: 3 lowerRightPiece: 4! !

!PocketCubeSideTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/17/2010 20:03'!
testCopyInLowerLayer

	side copyInLowerLayer: (Array with: 5 with: 6).
	self assert: side copyOutUpperLayer = (Array with:1 with:2).
	self assert: side copyOutLowerLayer = (Array with:5 with:6)! !

!PocketCubeSideTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/17/2010 20:01'!
testCopyInUpperLayer

	side copyInUpperLayer: (Array with: 5 with: 6).
	self assert: side copyOutUpperLayer = (Array with:5 with:6).
	self assert: side copyOutLowerLayer = (Array with:3 with:4)! !

!PocketCubeSideTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/17/2010 19:54'!
testCopyOutLowerLayer

	|twoPieces| 
	twoPieces := side copyOutLowerLayer.
	self assert: twoPieces = (Array with:3 with:4)! !

!PocketCubeSideTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/17/2010 19:53'!
testCopyOutUpperLayer

	|twoPieces| 
	twoPieces := side copyOutUpperLayer.
	self assert: twoPieces = (Array with:1 with:2)! !

!PocketCubeSideTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 01:31'!
testEqual

	self assert: side = (PocketCubeSide upperLeftPiece: 1 upperRightPiece: 2 lowerLeftPiece: 3 lowerRightPiece: 4).
	self deny: side = (PocketCubeSide upperLeftPiece: 1 upperRightPiece: 6 lowerLeftPiece: 3 lowerRightPiece: 4).
	self deny: side = (PocketCubeSide upperLeftPiece: 0 upperRightPiece: 2 lowerLeftPiece: 3 lowerRightPiece: 4).
	self deny: side = (PocketCubeSide upperLeftPiece: 1 upperRightPiece: 2 lowerLeftPiece: 9 lowerRightPiece: 4).
	self deny: side = (PocketCubeSide upperLeftPiece: 1 upperRightPiece: 2 lowerLeftPiece: 3 lowerRightPiece: 7).
! !

!PocketCubeSideTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 01:32'!
testHash

	self assert: side hash = (PocketCubeSide upperLeftPiece: 1 upperRightPiece: 2 lowerLeftPiece: 3 lowerRightPiece: 4) hash.
	self deny: side hash = (PocketCubeSide upperLeftPiece: 1 upperRightPiece: 6 lowerLeftPiece: 3 lowerRightPiece: 4) hash.
	self deny: side hash = (PocketCubeSide upperLeftPiece: 0 upperRightPiece: 2 lowerLeftPiece: 3 lowerRightPiece: 4) hash.
	self deny: side hash = (PocketCubeSide upperLeftPiece: 1 upperRightPiece: 2 lowerLeftPiece: 9 lowerRightPiece: 4) hash.
	self deny: side hash = (PocketCubeSide upperLeftPiece: 1 upperRightPiece: 2 lowerLeftPiece: 3 lowerRightPiece: 7) hash.
! !

!PocketCubeSideTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:54'!
testPrintPieces

	self assert: side printPieces = 
'1 2
3 4'! !

!PocketCubeSideTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/17/2010 19:35'!
testRotateClockwise

	side rotateClockwise.
	self assert: side upperLeftPiece = 3.
	self assert: side upperRightPiece = 1.
	self assert: side lowerLeftPiece = 4.
	self assert: side lowerRightPiece = 2! !

!PocketCubeSideTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/17/2010 19:46'!
testRotateCounterClockwise

	side rotateCounterClockwise.
	self assert: side upperLeftPiece = 2.
	self assert: side upperRightPiece = 4.
	self assert: side lowerLeftPiece = 1.
	self assert: side lowerRightPiece = 3! !

!PocketCubeSideTest methodsFor: 'as yet unclassified' stamp: 'hjo 7/17/2010 19:37'!
testSetup

	self assert: side upperLeftPiece = 1.
	self assert: side upperRightPiece = 2.
	self assert: side lowerLeftPiece = 3.
	self assert: side lowerRightPiece = 4! !

TestCase subclass: #PocketCubeWithMemoryTest
	instanceVariableNames: 'pocketCube'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-Tests'!

!PocketCubeWithMemoryTest methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 16:21'!
setUp

	super setUp.
	pocketCube := PocketCubeWithMemory new.
	pocketCube initializeSidesWithColors.! !

!PocketCubeWithMemoryTest methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 23:29'!
testAllOperations

	pocketCube twistUpperLayerRight.
	pocketCube rotateRight.
	pocketCube twistLowerLayerRight.
	pocketCube rotateLeft.
	pocketCube twistUpperLayerLeft.
	pocketCube rotateClockwise.
	pocketCube twistLowerLayerLeft.
	pocketCube rotateCounterClockwise.
	self assert: ({PoTwistUpperLayerRight. PoRotateRight. PoTwistLowerLayerRight. PoRotateLeft. PoTwistUpperLayerLeft. PoRotateClockwise. PoTwistLowerLayerLeft. PoRotateCounterClockwise} collect: [:each| each singleton]) equals: pocketCube allOperations.! !

!PocketCubeWithMemoryTest methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 12:14'!
testInverseOperation

	pocketCube twistUpperLayerLeft.
	pocketCube rotateRight.
	
	self assert: PoRotateLeft singleton equals: pocketCube firstInverseOperation! !

!PocketCubeWithMemoryTest methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 23:30'!
testRotateClockwise

	pocketCube rotateClockwise.
	self assert: PoRotateClockwise singleton equals: pocketCube lastOperation.! !

!PocketCubeWithMemoryTest methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 23:30'!
testRotateCounterClockwise

	pocketCube rotateCounterClockwise.
	self assert: PoRotateCounterClockwise singleton equals: pocketCube lastOperation.! !

!PocketCubeWithMemoryTest methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 23:31'!
testRotateLeft

	pocketCube rotateLeft.
	self assert: PoRotateLeft singleton equals: pocketCube lastOperation.! !

!PocketCubeWithMemoryTest methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 23:31'!
testRotateRight

	pocketCube rotateRight.
	self assert: PoRotateRight singleton equals: pocketCube lastOperation.! !

!PocketCubeWithMemoryTest methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 23:31'!
testTwistLowerLayerLeft

	pocketCube twistLowerLayerLeft.
	self assert: PoTwistLowerLayerLeft singleton equals: pocketCube lastOperation.! !

!PocketCubeWithMemoryTest methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 23:32'!
testTwistLowerLayerRight

	pocketCube twistLowerLayerRight.
	self assert: PoTwistLowerLayerRight singleton equals: pocketCube lastOperation.! !

!PocketCubeWithMemoryTest methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 23:32'!
testTwistUpperLayerLeft

	pocketCube twistUpperLayerLeft.
	self assert: PoTwistUpperLayerLeft singleton equals: pocketCube lastOperation.! !

!PocketCubeWithMemoryTest methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 23:31'!
testTwistUpperLayerRight

	pocketCube twistUpperLayerRight.
	self assert: PoTwistUpperLayerRight singleton equals: pocketCube lastOperation.! !

TestCase subclass: #SidePatternTest
	instanceVariableNames: 'pocketCubeSide'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-SidePatterns-Tests'!

!SidePatternTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 21:22'!
setUp

	super setUp.! !

!SidePatternTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 21:25'!
testBottomBarSidePattern

	| pattern |
	pocketCubeSide := PocketCubeSide upperLeftPiece: Color yellow  upperRightPiece: Color blue lowerLeftPiece: Color green lowerRightPiece: Color green.
	pattern := pocketCubeSide sidePattern.
	self assert: BottomBarSidePattern equals: pattern class.
	self assert: Color green equals: pattern barColor.
	self assert: Color yellow equals: pattern colorOfLeftToTheBar.
	self assert: Color blue equals: pattern colorOfRightToTheBar.! !

!SidePatternTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/4/2011 14:30'!
testBottomLeftL

	| pattern |
	pocketCubeSide := PocketCubeSide upperLeftPiece: Color blue upperRightPiece: Color red lowerLeftPiece: Color blue lowerRightPiece: Color blue.
	pattern := pocketCubeSide sidePattern.
	self assert: BottomLeftLSidePattern equals: pattern class.
	self assert: Color blue equals: pattern colorOfL.
	self assert: Color red equals: pattern colorOfDot.

! !

!SidePatternTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/4/2011 20:49'!
testBottomRightL

	| pattern |
	pocketCubeSide := PocketCubeSide upperLeftPiece: Color red upperRightPiece: Color blue lowerLeftPiece: Color blue lowerRightPiece: Color blue.
	pattern := pocketCubeSide sidePattern.
	self assert: BottomRightLSidePattern equals: pattern class.
	self assert: Color blue equals: pattern colorOfL.
	self assert: Color red equals: pattern colorOfDot.

! !

!SidePatternTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 13:35'!
testFourColor

	| pattern |
	pocketCubeSide := PocketCubeSide upperLeftPiece: Color yellow  upperRightPiece: Color blue lowerLeftPiece: Color green lowerRightPiece: Color red.
	pattern := pocketCubeSide sidePattern.
	self assert: FourColorSidePattern equals: pattern class.
	self assert: pocketCubeSide equals: pattern side.! !

!SidePatternTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/4/2011 13:34'!
testHorizontallyParallel

	| pattern |
	pocketCubeSide := PocketCubeSide upperLeftPiece: Color white upperRightPiece: Color white lowerLeftPiece: Color green lowerRightPiece: Color green.
	pattern := pocketCubeSide sidePattern.
	self assert: HorizontallyParallelSidePattern equals: pattern class.
	self assert: Color white equals: pattern colorOfTopBar.
	self assert: Color green equals: pattern colorOfBottomBar.

! !

!SidePatternTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 21:59'!
testLeftBarSidePattern

	| pattern |
	pocketCubeSide := PocketCubeSide upperLeftPiece: Color yellow  upperRightPiece: Color blue lowerLeftPiece: Color yellow lowerRightPiece: Color green.
	pattern := pocketCubeSide sidePattern.
	self assert: LeftBarSidePattern equals: pattern class.
	self assert: Color yellow equals: pattern barColor.
	self assert: Color blue equals: pattern colorOfLeftToTheBar.
	self assert: Color green equals: pattern colorOfRightToTheBar.! !

!SidePatternTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 22:27'!
testRightBarSidePattern

	| pattern |
	pocketCubeSide := PocketCubeSide upperLeftPiece: Color blue upperRightPiece: Color yellow lowerLeftPiece: Color green lowerRightPiece: Color yellow.
	pattern := pocketCubeSide sidePattern.
	self assert: RightBarSidePattern equals: pattern class.
	self assert: Color yellow equals: pattern barColor.
	self assert: Color green equals: pattern colorOfLeftToTheBar.
	self assert: Color blue equals: pattern colorOfRightToTheBar.! !

!SidePatternTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 22:35'!
testTopBarSidePattern

	| pattern |
	pocketCubeSide := PocketCubeSide upperLeftPiece: Color orange upperRightPiece: Color orange lowerLeftPiece: Color green lowerRightPiece: Color yellow.
	pattern := pocketCubeSide sidePattern.
	self assert: TopBarSidePattern equals: pattern class.
	self assert: Color orange equals: pattern barColor.
	self assert: Color yellow equals: pattern colorOfLeftToTheBar.
	self assert: Color green equals: pattern colorOfRightToTheBar.! !

!SidePatternTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 12:17'!
testTopLeftL

	| pattern |
	pocketCubeSide := PocketCubeSide upperLeftPiece: Color green upperRightPiece: Color green lowerLeftPiece: Color green lowerRightPiece: Color red.
	pattern := pocketCubeSide sidePattern.
	self assert: TopLeftLSidePattern equals: pattern class.
	self assert: Color green equals: pattern colorOfL.
	self assert: Color red equals: pattern colorOfDot.

! !

!SidePatternTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 12:13'!
testTopRightL

	| pattern |
	pocketCubeSide := PocketCubeSide upperLeftPiece: Color green upperRightPiece: Color green lowerLeftPiece: Color red lowerRightPiece: Color green.
	pattern := pocketCubeSide sidePattern.
	self assert: TopRightLSidePattern equals: pattern class.
	self assert: Color green equals: pattern colorOfL.
	self assert: Color red equals: pattern colorOfDot.

! !

!SidePatternTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/4/2011 13:39'!
testUnifiedSidePattern

	| pattern |
	pocketCubeSide := PocketCubeSide upperLeftPiece: Color green  upperRightPiece: Color green lowerLeftPiece: Color green lowerRightPiece: Color green.
	pattern := pocketCubeSide sidePattern.
	self assert: UnifiedSidePattern equals: pattern class.
	self assert: Color green equals: pattern color.! !

!SidePatternTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/4/2011 13:46'!
testVerticallyParallel

	| pattern |
	pocketCubeSide := PocketCubeSide upperLeftPiece: Color blue upperRightPiece: Color red lowerLeftPiece: Color blue lowerRightPiece: Color red.
	pattern := pocketCubeSide sidePattern.
	self assert: VerticallyParallelSidePattern equals: pattern class.
	self assert: Color blue equals: pattern colorOfLeftBar.
	self assert: Color red equals: pattern colorOfRightBar.

! !

Object subclass: #PocketCube
	instanceVariableNames: 'topSide leftSide bottomSide rightSide frontSide backSide'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube'!

!PocketCube methodsFor: 'sides' stamp: 'hjo 1/31/2011 19:16'!
allSides

	^Array streamContents: [:stream| 
		stream nextPut: topSide.
		stream nextPut: leftSide.
		stream nextPut: bottomSide.
		stream nextPut: rightSide.
		stream nextPut: frontSide.
		stream nextPut: backSide  ]! !

!PocketCube methodsFor: 'testing' stamp: 'hjo 1/31/2011 19:14'!
allSidesAreOneColor

	^self allSides allSatisfy: [:each| each isOneColor ] ! !

!PocketCube methodsFor: 'rotate' stamp: 'hjo 7/18/2010 22:53'!
arrayOfBackSide

	^Array with: backSide! !

!PocketCube methodsFor: 'sides' stamp: 'hjo 7/17/2010 20:31'!
backSide

	^backSide! !

!PocketCube methodsFor: 'sides' stamp: 'hjo 7/17/2010 20:32'!
backSide: aPocketCubeSide 

	backSide := aPocketCubeSide ! !

!PocketCube methodsFor: 'sides' stamp: 'hjo 7/17/2010 20:23'!
bottomSide

	^bottomSide! !

!PocketCube methodsFor: 'sides' stamp: 'hjo 7/17/2010 20:22'!
bottomSide: aPocketCubeSide 

	bottomSide := aPocketCubeSide ! !

!PocketCube methodsFor: 'sides' stamp: 'hjo 7/17/2010 20:29'!
frontSide

	^frontSide! !

!PocketCube methodsFor: 'sides' stamp: 'hjo 7/17/2010 20:30'!
frontSide: aPocketCubeSide 

	frontSide := aPocketCubeSide ! !

!PocketCube methodsFor: 'testing' stamp: 'hjo 2/2/2011 21:46'!
frontSideBackSideAreOneColor

	^frontSide isOneColor and: [backSide isOneColor]! !

!PocketCube methodsFor: 'initializing' stamp: 'hjo 2/2/2011 18:57'!
initializeSidesWithColors

	self topSide: (PocketCubeSide ofAllSamePieces: Color red).
	self leftSide: (PocketCubeSide ofAllSamePieces: Color white).
	self bottomSide: (PocketCubeSide ofAllSamePieces: Color orange).
	self rightSide: (PocketCubeSide ofAllSamePieces: Color yellow).
	self frontSide: (PocketCubeSide ofAllSamePieces: Color green).
	self backSide: (PocketCubeSide ofAllSamePieces: Color blue).! !

!PocketCube methodsFor: 'sides' stamp: 'hjo 7/17/2010 20:19'!
leftSide

	^leftSide! !

!PocketCube methodsFor: 'sides' stamp: 'hjo 7/17/2010 20:19'!
leftSide: aPocketCubeSide 

	leftSide := aPocketCubeSide! !

!PocketCube methodsFor: 'collection' stamp: 'hjo 2/21/2011 11:11'!
pieceSet
	
	^self allSides inject: Set new into: [  :set :each | set addAll: each pieceSet. set ]! !

!PocketCube methodsFor: 'sides' stamp: 'hjo 7/17/2010 20:26'!
rightSide

	^rightSide! !

!PocketCube methodsFor: 'sides' stamp: 'hjo 7/17/2010 20:27'!
rightSide: aPocketCubeSide 

	rightSide := aPocketCubeSide! !

!PocketCube methodsFor: 'rotate' stamp: 'hjo 2/1/2011 18:49'!
rotateClockwise

	self rotateClockwiseForSideAroundFrontFace.
	self rotateSidesClockwise: self sidesWithoutBackSide 
		rotateSidesCounterClockwise: self arrayOfBackSide.
	self triggerEvent: #pocketCubeChanged.! !

!PocketCube methodsFor: 'rotate' stamp: 'hjo 7/18/2010 22:43'!
rotateClockwiseForSideAroundFrontFace

	self setSidesTopRightBottomLeft: 
		(Array 
			with: leftSide 
			with: topSide 
			with: rightSide 
			with: bottomSide)
! !

!PocketCube methodsFor: 'rotate' stamp: 'hjo 2/1/2011 18:49'!
rotateCounterClockwise

	self rotateCounterClockwiseForSideAroundFrontFace.
	self rotateSidesClockwise: self arrayOfBackSide 
		rotateSidesCounterClockwise: self sidesWithoutBackSide.
	self triggerEvent: #pocketCubeChanged.! !

!PocketCube methodsFor: 'rotate' stamp: 'hjo 7/18/2010 22:44'!
rotateCounterClockwiseForSideAroundFrontFace

	self setSidesTopRightBottomLeft:
		(Array
			with: rightSide
			with: bottomSide
			with: leftSide
			with: topSide)! !

!PocketCube methodsFor: 'rotate' stamp: 'hjo 3/24/2011 17:19'!
rotateLeft

	topSide rotateCounterClockwise.
	bottomSide rotateClockwise.
	self twistUpperLayerWithSides: self sidesReverseOrder.
	self twistLowerLayerWithSides: self sidesReverseOrder.
	self triggerEvent: #pocketCubeChanged.! !

!PocketCube methodsFor: 'rotate' stamp: 'hjo 7/18/2010 07:49'!
rotateLeftForSides

	| tempSide |
	tempSide := backSide.
	backSide := rightSide.
	rightSide := frontSide.
	frontSide := leftSide.
	leftSide := tempSide! !

!PocketCube methodsFor: 'rotate' stamp: 'hjo 3/24/2011 17:17'!
rotateRight

	topSide rotateClockwise.
	bottomSide rotateCounterClockwise.
	self twistUpperLayerWithSides: self sidesInOrder.
	self twistLowerLayerWithSides: self sidesInOrder.
	self triggerEvent: #pocketCubeChanged.
! !

!PocketCube methodsFor: 'rotate' stamp: 'hjo 7/18/2010 09:44'!
rotateRightForSides

	| tempSide |
	tempSide := leftSide.
	leftSide := frontSide.
	frontSide := rightSide.
	rightSide := backSide.
	backSide := tempSide! !

!PocketCube methodsFor: 'rotate' stamp: 'hjo 7/18/2010 22:50'!
rotateSidesClockwise: anArray rotateSidesCounterClockwise: anArray2 

	anArray do: [:each| each rotateClockwise ].
	anArray2 do: [:each| each rotateCounterClockwise ].! !

!PocketCube methodsFor: 'rotate' stamp: 'hjo 7/18/2010 22:41'!
setSidesTopRightBottomLeft: anArray

	topSide := anArray at: 1.
	rightSide := anArray at: 2.
	bottomSide := anArray at: 3.
	leftSide := anArray at: 4! !

!PocketCube methodsFor: 'collection' stamp: 'hjo 2/7/2011 10:22'!
sideColumn

	^(Array with: topSide with: frontSide with: bottomSide with: self virtualVerticalBackSide)
		collect: [:each| each rotatedCounterClockwisely]! !

!PocketCube methodsFor: 'collection' stamp: 'hjo 2/6/2011 19:23'!
sideRow

	^self sidesInOrder! !

!PocketCube methodsFor: 'testing' stamp: 'hjo 2/2/2011 19:30'!
sidesAreUpperLayerTwisted: array1 to: array2

	1 to: 4 do: [:index| 
		(self sidesInOrder at:(array1 at:index)) copyOutLowerLayer = (self sidesInOrder at:(array2 at:index)) copyOutUpperLayer
			ifFalse: [^false].
		].
	^true! !

!PocketCube methodsFor: 'testing' stamp: 'hjo 2/2/2011 19:27'!
sidesAreUpperLayerTwistedLeft

	^self sidesAreUpperLayerTwisted: #(2 3 4 1) to: #(1 2 3 4).! !

!PocketCube methodsFor: 'testing' stamp: 'hjo 2/2/2011 19:27'!
sidesAreUpperLayerTwistedRight

	^self sidesAreUpperLayerTwisted: #(1 2 3 4) to: #(2 3 4 1).! !

!PocketCube methodsFor: 'testing' stamp: 'hjo 2/2/2011 19:28'!
sidesAreUpperLayerTwistedTwice

	^self sidesAreUpperLayerTwisted: #(1 2 3 4) to: #(3 4 1 2).! !

!PocketCube methodsFor: 'twist' stamp: 'hjo 7/17/2010 22:38'!
sidesInOrder

	^Array with: leftSide with: frontSide with: rightSide with:backSide! !

!PocketCube methodsFor: 'twist' stamp: 'hjo 2/1/2011 21:57'!
sidesReverseOrder

	^self sidesInOrder reversed! !

!PocketCube methodsFor: 'rotate' stamp: 'hjo 7/18/2010 22:55'!
sidesWithoutBackSide
	
	^Array with: frontSide with: topSide with: leftSide with: rightSide with: bottomSide! !

!PocketCube methodsFor: 'testing' stamp: 'hjo 2/1/2011 18:46'!
slidesAreVerticallyLayerTwisted

	self rotateClockwise.
	[self sidesAreUpperLayerTwistedLeft ifTrue: [^true].
	self sidesAreUpperLayerTwistedRight ifTrue: [^true].
	self sidesAreUpperLayerTwistedTwice ifTrue: [^true].
	^false] ensure: [self rotateCounterClockwise].
	! !

!PocketCube methodsFor: 'sides' stamp: 'hjo 7/17/2010 20:15'!
topSide

	^topSide! !

!PocketCube methodsFor: 'sides' stamp: 'hjo 7/17/2010 20:15'!
topSide: aPocketCubeSide 

	topSide := aPocketCubeSide! !

!PocketCube methodsFor: 'twist' stamp: 'hjo 2/1/2011 18:47'!
twistLowerLayerLeft

	bottomSide rotateCounterClockwise.
	self twistLowerLayerWithSides: self sidesInOrder.
	self triggerEvent: #pocketCubeChanged.! !

!PocketCube methodsFor: 'twist' stamp: 'hjo 2/1/2011 12:57'!
twistLowerLayerRight

	bottomSide rotateClockwise.
	self twistLowerLayerWithSides: self sidesReverseOrder.
	self triggerEvent: #pocketCubeChanged! !

!PocketCube methodsFor: 'twist' stamp: 'hjo 7/17/2010 22:32'!
twistLowerLayerWithSides: anArray 

	| twoPieces |
	twoPieces := (anArray at:1) copyOutLowerLayer.
	(anArray at:1) copyInLowerLayer: (anArray at:2) copyOutLowerLayer.
	(anArray at:2) copyInLowerLayer: (anArray at:3) copyOutLowerLayer.
	(anArray at:3) copyInLowerLayer: (anArray at:4) copyOutLowerLayer.
	(anArray at:4) copyInLowerLayer: twoPieces! !

!PocketCube methodsFor: 'twist' stamp: 'hjo 2/1/2011 18:48'!
twistUpperLayerLeft

	topSide rotateClockwise.
	self twistUpperLayerWithSides: self sidesInOrder.
	self triggerEvent: #pocketCubeChanged.! !

!PocketCube methodsFor: 'twist' stamp: 'hjo 2/1/2011 18:48'!
twistUpperLayerRight

	topSide rotateCounterClockwise.
	self twistUpperLayerWithSides: self sidesReverseOrder.
	self triggerEvent: #pocketCubeChanged.! !

!PocketCube methodsFor: 'twist' stamp: 'hjo 7/17/2010 22:31'!
twistUpperLayerWithSides: anArray 

	| twoPieces |
	twoPieces := (anArray at:1) copyOutUpperLayer.
	(anArray at:1) copyInUpperLayer: (anArray at:2) copyOutUpperLayer.
	(anArray at:2) copyInUpperLayer: (anArray at:3) copyOutUpperLayer.
	(anArray at:3) copyInUpperLayer: (anArray at:4) copyOutUpperLayer.
	(anArray at:4) copyInUpperLayer: twoPieces! !

!PocketCube methodsFor: 'twist' stamp: 'hjo 2/3/2011 15:05'!
verticalSides

	^Array with: topSide with: frontSide  with: bottomSide  with: backSide! !

!PocketCube methodsFor: 'sides' stamp: 'hjo 2/5/2011 21:12'!
virtualVerticalBackSide

	^PocketCubeSide upperLeftPiece: backSide lowerRightPiece upperRightPiece: backSide lowerLeftPiece  lowerLeftPiece: backSide upperRightPiece  lowerRightPiece: backSide upperLeftPiece
	! !

PocketCube subclass: #PocketCubeWithMemory
	instanceVariableNames: 'moveHistory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube'!

!PocketCubeWithMemory methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 23:33'!
addToMoveHistory: aPocketCubeOperation

	moveHistory addLast: aPocketCubeOperation! !

!PocketCubeWithMemory methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 17:13'!
allOperations

	^moveHistory asArray! !

!PocketCubeWithMemory methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 12:20'!
firstInverseOperation

	^self lastOperation inverseOperation! !

!PocketCubeWithMemory methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 12:48'!
forgetLastOperation

	moveHistory isEmpty ifTrue: [^self "do nothing"].
	moveHistory removeLast! !

!PocketCubeWithMemory methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 15:44'!
initialize

	super initialize.
	moveHistory := OrderedCollection new.! !

!PocketCubeWithMemory methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 16:21'!
lastOperation

	^moveHistory last! !

!PocketCubeWithMemory methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 18:03'!
pocketCubeIsModified
	
	^moveHistory notEmpty! !

!PocketCubeWithMemory methodsFor: 'rotate' stamp: 'hjo 3/24/2011 23:28'!
rotateClockwise

	super rotateClockwise.
	self addToMoveHistory: PoRotateClockwise singleton! !

!PocketCubeWithMemory methodsFor: 'rotate' stamp: 'hjo 3/24/2011 23:28'!
rotateCounterClockwise

	super rotateCounterClockwise.
	self addToMoveHistory: PoRotateCounterClockwise singleton! !

!PocketCubeWithMemory methodsFor: 'rotate' stamp: 'hjo 3/24/2011 23:28'!
rotateLeft

	super rotateLeft.
	self addToMoveHistory: PoRotateLeft singleton! !

!PocketCubeWithMemory methodsFor: 'rotate' stamp: 'hjo 3/24/2011 23:29'!
rotateRight

	super rotateRight.
	self addToMoveHistory: PoRotateRight singleton! !

!PocketCubeWithMemory methodsFor: 'twist' stamp: 'hjo 3/24/2011 23:29'!
twistLowerLayerLeft

	super twistLowerLayerLeft.
	self addToMoveHistory: PoTwistLowerLayerLeft singleton! !

!PocketCubeWithMemory methodsFor: 'twist' stamp: 'hjo 3/24/2011 23:29'!
twistLowerLayerRight

	super twistLowerLayerRight.
	self addToMoveHistory: PoTwistLowerLayerRight singleton! !

!PocketCubeWithMemory methodsFor: 'twist' stamp: 'hjo 3/24/2011 23:29'!
twistUpperLayerLeft

	super twistUpperLayerLeft.
	self addToMoveHistory: PoTwistUpperLayerLeft singleton! !

!PocketCubeWithMemory methodsFor: 'twist' stamp: 'hjo 3/24/2011 23:29'!
twistUpperLayerRight

	super twistUpperLayerRight.
	self addToMoveHistory: PoTwistUpperLayerRight singleton! !

Object subclass: #PocketCubeController
	instanceVariableNames: 'model frontSideController topSideController bottomSideController leftSideController rightSideController backSideController virtualVerticalBackSideController gameMorph moveCubeSound'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-Morph'!

!PocketCubeController methodsFor: 'as yet unclassified' stamp: 'hjo 2/7/2011 11:59'!
beScrambled: aSmallInteger

	(PocketCubeScrambler new netStyleScrambleSequenceOf: aSmallInteger) do: [:each| self perform: each]! !

!PocketCubeController methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 12:38'!
deleteMorph

	gameMorph delete! !

!PocketCubeController methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 12:38'!
focused

	ActiveHand keyboardFocus: gameMorph! !

!PocketCubeController methodsFor: 'as yet unclassified' stamp: 'hjo 2/7/2011 10:26'!
horizontalRowY

	^145! !

!PocketCubeController methodsFor: 'as yet unclassified' stamp: 'hjo 2/7/2011 10:26'!
initialize

	super initialize.
	self initializeModel.
	gameMorph := PocketCubeGameMorph new.
	gameMorph controller: self.
		
	frontSideController := self makeSideControllerWithSelector: #frontSide atOrigin: self verticalColumnX@self horizontalRowY.
	topSideController := self makeSideControllerWithSelector: #topSide atOrigin: self verticalColumnX@55.
	bottomSideController := self makeSideControllerWithSelector: #bottomSide atOrigin: self verticalColumnX@235.
	leftSideController := self makeSideControllerWithSelector: #leftSide atOrigin: 50@self horizontalRowY.
	rightSideController := self makeSideControllerWithSelector: #rightSide atOrigin: 230@self horizontalRowY.
	backSideController := self makeSideControllerWithSelector: #backSide atOrigin: 320@self horizontalRowY.
	virtualVerticalBackSideController := self makeVirtualSideControllerWithSelector: #virtualVerticalBackSide atOrigin: self verticalColumnX@325.

	moveCubeSound := SampledSound fromAIFFfileNamed: 'cube-move.aiff'.! !

!PocketCubeController methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 15:51'!
initializeModel

	model := PocketCubeWithMemory new.
	model when: #pocketCubeChanged send: #refreshMorph to: self. 
	model initializeSidesWithColors.! !

!PocketCubeController methodsFor: 'as yet unclassified' stamp: 'hjo 2/6/2011 13:03'!
makeSideControllerWithSelector: aSymbol atOrigin: aPoint

	^PocketCubeSideController openMorphOn: model sideAccessor: aSymbol atOrigin: aPoint gameMorph: gameMorph.! !

!PocketCubeController methodsFor: 'as yet unclassified' stamp: 'hjo 2/6/2011 13:09'!
makeVirtualSideControllerWithSelector: aByteSymbol atOrigin: aPoint 

	^PocketCubeVirtualSideController openMorphOn: model sideAccessor: aByteSymbol atOrigin: aPoint gameMorph: gameMorph.! !

!PocketCubeController methodsFor: 'accessing' stamp: 'hjo 2/21/2011 11:05'!
model
	
	^model! !

!PocketCubeController methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 15:29'!
model: aPocketCube 

	model removeActionsWithReceiver: self.
	model := aPocketCube.
	model when: #pocketCubeChanged send: #refreshMorph to: self. 
	topSideController pocketCube: model.
	bottomSideController pocketCube: model.
	leftSideController pocketCube: model.
	rightSideController pocketCube: model.
	backSideController pocketCube: model.
	virtualVerticalBackSideController pocketCube: model.
	self refreshMorph.! !

!PocketCubeController methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 20:49'!
playSoundMoveCube

	moveCubeSound play.! !

!PocketCubeController methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 12:38'!
position: aPoint 

	gameMorph position: aPoint! !

!PocketCubeController methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 15:29'!
refreshMorph

	frontSideController refreshMorph.
	backSideController refreshMorph.
	leftSideController refreshMorph.
	rightSideController refreshMorph.
	topSideController refreshMorph.
	bottomSideController refreshMorph.
	virtualVerticalBackSideController refreshMorph.! !

!PocketCubeController methodsFor: 'rotate' stamp: 'hjo 2/1/2011 21:44'!
rotateClockwise
	
	model rotateClockwise.! !

!PocketCubeController methodsFor: 'rotate' stamp: 'hjo 2/1/2011 21:45'!
rotateCounterClockwise

	model rotateCounterClockwise.! !

!PocketCubeController methodsFor: 'rotate' stamp: 'hjo 2/1/2011 21:45'!
rotateLeft
	model rotateLeft.! !

!PocketCubeController methodsFor: 'rotate' stamp: 'hjo 2/1/2011 21:46'!
rotateRight

	model rotateRight.! !

!PocketCubeController methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 12:38'!
setControllerForGameMorph
	
	gameMorph controller: self! !

!PocketCubeController methodsFor: 'as yet unclassified' stamp: 'hjo 1/31/2011 19:04'!
solve

	| recog |
	recog := PocketCubeRecognizer on: model.
	^recog! !

!PocketCubeController methodsFor: 'twist' stamp: 'hjo 2/5/2011 20:48'!
twistLowerLayerLeft

	model twistLowerLayerLeft.
	self playSoundMoveCube.! !

!PocketCubeController methodsFor: 'twist' stamp: 'hjo 2/5/2011 20:48'!
twistLowerLayerRight

	model twistLowerLayerRight.
	self playSoundMoveCube.! !

!PocketCubeController methodsFor: 'twist' stamp: 'hjo 2/5/2011 20:48'!
twistUpperLayerLeft

	model twistUpperLayerLeft.
	self playSoundMoveCube.! !

!PocketCubeController methodsFor: 'twist' stamp: 'hjo 2/5/2011 20:48'!
twistUpperLayerRight

	model twistUpperLayerRight.
	self playSoundMoveCube.! !

!PocketCubeController methodsFor: 'as yet unclassified' stamp: 'hjo 2/6/2011 13:12'!
verticalColumnX

	^140! !

Object subclass: #PocketCubeOperation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube'!

PocketCubeOperation class
	instanceVariableNames: 'SingletonObject'!

PocketCubeOperation class
	instanceVariableNames: 'SingletonObject'!

!PocketCubeOperation classSide methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 21:28'!
singleton

	SingletonObject ifNil: [SingletonObject := self basicNew].
	^SingletonObject! !

!PocketCubeOperation methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 11:03'!
applyTo: aPocketCube 

	^aPocketCube perform: self selector ! !

PocketCubeOperation subclass: #PocketCubeRotateOperation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube'!

PocketCubeRotateOperation subclass: #PoRotateClockwise
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube'!

!PoRotateClockwise methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 23:18'!
inverseOperation

	^PoRotateCounterClockwise singleton! !

!PoRotateClockwise methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 21:33'!
selector

	^#rotateClockwise! !

PocketCubeRotateOperation subclass: #PoRotateCounterClockwise
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube'!

!PoRotateCounterClockwise methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 23:19'!
inverseOperation

	^PoRotateClockwise singleton! !

!PoRotateCounterClockwise methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 21:51'!
selector

	^#rotateCounterClockwise! !

PocketCubeRotateOperation subclass: #PoRotateLeft
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube'!

!PoRotateLeft methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 23:17'!
inverseOperation

	^PoRotateRight singleton! !

!PoRotateLeft methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 21:54'!
selector

	^#rotateLeft! !

PocketCubeRotateOperation subclass: #PoRotateRight
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube'!

!PoRotateRight methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 23:19'!
inverseOperation

	^PoRotateLeft singleton! !

!PoRotateRight methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 21:58'!
selector

	^#rotateRight! !

PocketCubeOperation subclass: #PocketCubeTwistOperation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube'!

PocketCubeTwistOperation subclass: #PoTwistLowerLayerLeft
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube'!

!PoTwistLowerLayerLeft methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 23:20'!
inverseOperation

	^PoTwistLowerLayerRight singleton! !

!PoTwistLowerLayerLeft methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 22:02'!
selector
	^#twistLowerLayerLeft! !

PocketCubeTwistOperation subclass: #PoTwistLowerLayerRight
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube'!

!PoTwistLowerLayerRight methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 23:18'!
inverseOperation

	^PoTwistLowerLayerLeft singleton! !

!PoTwistLowerLayerRight methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 22:12'!
selector

	^#twistLowerLayerRight! !

PocketCubeTwistOperation subclass: #PoTwistUpperLayerLeft
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube'!

!PoTwistUpperLayerLeft methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 23:20'!
inverseOperation

	^PoTwistUpperLayerRight singleton! !

!PoTwistUpperLayerLeft methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 22:21'!
selector

	^#twistUpperLayerLeft! !

PocketCubeTwistOperation subclass: #PoTwistUpperLayerRight
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube'!

!PoTwistUpperLayerRight methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 23:21'!
inverseOperation

	^PoTwistUpperLayerLeft singleton! !

!PoTwistUpperLayerRight methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 22:26'!
selector

	^#twistUpperLayerRight! !

Object subclass: #PocketCubePattern
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-Recognizer'!

!PocketCubePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 15:39'!
removeSubclassPatterns

	self allSubclassesDo: [:each| 
		each = PocketCubeUnknownPattern
			ifFalse: [each removeFromSystem]].
	PocketCubeRecognizer removeAllPatternClasses.! !

PocketCubePattern subclass: #PocketCubeUnknownPattern
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-Recognizer'!

!PocketCubeUnknownPattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/1/2011 18:23'!
oneStepBetterFor: aPocketCube 

	self error: 'i do not know what to do. 
	please, 
		check #scanPattern.
		define new pattern class.'.! !

Object subclass: #PocketCubePatternGenerator
	instanceVariableNames: 'pocketCube scrambler newClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-Recognizer'!

!PocketCubePatternGenerator classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/21/2011 15:17'!
on: aPocketCube withScrambler: aPocketCubeScrambler 

	| instance |
	instance := self new.
	instance pocketCube: aPocketCube.
	instance scrambler: aPocketCubeScrambler.
	^instance! !

!PocketCubePatternGenerator methodsFor: 'as yet unclassified' stamp: 'hjo 2/21/2011 12:59'!
colorToMatches

	| colorToMatches |
	colorToMatches := Dictionary new.
	self pieceSet do: [:colorLookingFor| 
		| matches |
		matches := OrderedCollection new.
		#(topSide frontSide bottomSide leftSide rightSide backSide) do: 
			[:sideGetter |
				#(upperLeftPiece upperRightPiece lowerLeftPiece lowerRightPiece) do: 
					[:pieceGetter | |pieceGot|
						pieceGot := (pocketCube perform: sideGetter) perform: pieceGetter.
						pieceGot = colorLookingFor
							ifTrue: [ matches add: {sideGetter . pieceGetter}].
					]].
		colorToMatches at: colorLookingFor put: matches.
	].
	^colorToMatches ! !

!PocketCubePatternGenerator methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 19:23'!
generatePatternClass

	^scrambler pocketCubeIsModified
		ifTrue: [self generatePatternClassForModifiedPocketCube]
		ifFalse: [self generatePatternClassForUnmodifiedPocketCube]
	! !

!PocketCubePatternGenerator methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 15:13'!
generatePatternClassForModifiedPocketCube
	newClass := self primGeneratePatternClass.
	newClass class compile: self implementationStringOfIsPatternOf.
	newClass class compile: self implementationStringOfOneStepBetterWithAction.
	^ newClass! !

!PocketCubePatternGenerator methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 15:12'!
generatePatternClassForUnmodifiedPocketCube
	newClass := self primGeneratePatternClass.
	newClass class compile: self implementationStringOfIsPatternOf.
	newClass class compile: self implementationStringOfOneStepBetterDoNothing.
	^ newClass! !

!PocketCubePatternGenerator methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 15:08'!
implementationStringOfIsPatternOf
	^ 'isPatternOf: aPocketCube

' , self patternMatchingCode! !

!PocketCubePatternGenerator methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 15:09'!
implementationStringOfOneStepBetterDoNothing
	^ 'oneStepBetterFor: aPocketCube 

	"do nothing"'! !

!PocketCubePatternGenerator methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 15:36'!
implementationStringOfOneStepBetterWithAction
	^ 'oneStepBetterFor: aPocketCube 

	aPocketCube ' , scrambler firstInverseOperation selector , '.
	aPocketCube forgetLastOperation.
	aPocketCube forgetLastOperation.'! !

!PocketCubePatternGenerator methodsFor: 'as yet unclassified' stamp: 'hjo 2/21/2011 13:46'!
patternMatchingCode

	|codeStream|
	codeStream := WriteStream on: ''.
	codeStream nextPutAll: '^'.
	self colorToMatches values do: [:group| 
		codeStream nextPutAll: '(Set'.
		group do: [:each| 
			codeStream nextPutAll: ' with: aPocketCube '.
			codeStream nextPutAll: each first.
			codeStream space.
			codeStream nextPutAll: each second
			].
			codeStream nextPutAll: ') size = 1'.
		]
		separatedBy: [ codeStream nextPutAll: ' and: ['. codeStream cr ].
	codeStream nextPutAll: ']]]]]'.
	^codeStream contents! !

!PocketCubePatternGenerator methodsFor: 'as yet unclassified' stamp: 'hjo 2/21/2011 13:01'!
pieceSet
	
	^pocketCube pieceSet.! !

!PocketCubePatternGenerator methodsFor: 'as yet unclassified' stamp: 'hjo 2/21/2011 11:06'!
pocketCube: aPocketCube 
	
	pocketCube := aPocketCube ! !

!PocketCubePatternGenerator methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 14:56'!
primGeneratePatternClass
	^ PocketCubePattern
		subclass: ('PocketCubeMechanical' , PocketCubePattern allSubclasses size printString , 'Pattern') asSymbol
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: PocketCubePattern category asString! !

!PocketCubePatternGenerator methodsFor: 'as yet unclassified' stamp: 'hjo 2/21/2011 15:16'!
scrambler: aPocketCubeScrambler 

	scrambler := aPocketCubeScrambler ! !

Object subclass: #PocketCubeRecognizer
	instanceVariableNames: 'pocketCube pattern'
	classVariableNames: 'PatternClasses'
	poolDictionaries: ''
	category: 'PocketCube-Recognizer'!

!PocketCubeRecognizer classSide methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 16:35'!
initialize 

	self removeAllPatternClasses! !

!PocketCubeRecognizer classSide methodsFor: 'as yet unclassified' stamp: 'hjo 1/31/2011 19:05'!
on: aPocketCube 

	|instance|
	instance := self new.
	instance pocketCube: aPocketCube.
	^instance.! !

!PocketCubeRecognizer classSide methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 15:40'!
removeAllPatternClasses

	PatternClasses := OrderedCollection new! !

!PocketCubeRecognizer methodsFor: 'as yet unclassified' stamp: 'hjo 2/1/2011 12:54'!
oneStepBetter

	self scanPattern.
	self oneStepBetterWithPattern.
	pocketCube triggerEvent: #pocketCubeChanged! !

!PocketCubeRecognizer methodsFor: 'as yet unclassified' stamp: 'hjo 2/1/2011 12:57'!
oneStepBetterWithPattern

	pattern oneStepBetterFor: pocketCube.! !

!PocketCubeRecognizer methodsFor: 'accessing' stamp: 'hjo 2/2/2011 23:58'!
pattern: aClass 

	pattern := aClass! !

!PocketCubeRecognizer methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 12:11'!
patternFound

	^pattern notNil! !

!PocketCubeRecognizer methodsFor: 'accessing' stamp: 'hjo 2/1/2011 12:53'!
pocketCube: aPocketCube 

	pocketCube := aPocketCube.! !

!PocketCubeRecognizer methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 19:27'!
scanPattern

	pattern := nil.
	PatternClasses do: [:each| self scanPattern: each].
	pattern ifNil: [
		pattern := (PocketCubePatternGenerator on: pocketCube withScrambler: pocketCube) generatePatternClass.
		PatternClasses addLast: pattern ].
	^pattern! !

!PocketCubeRecognizer methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 12:01'!
scanPattern: aPattern

	self patternFound ifFalse: [
		(aPattern isPatternOf: pocketCube) ifTrue: [self pattern: aPattern]].! !

Object subclass: #PocketCubeScrambleOperation
	instanceVariableNames: 'amount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube'!

!PocketCubeScrambleOperation methodsFor: 'as yet unclassified' stamp: 'hjo 2/7/2011 12:15'!
amount 

	^amount! !

!PocketCubeScrambleOperation methodsFor: 'as yet unclassified' stamp: 'hjo 2/7/2011 12:15'!
amount: anInteger

	amount := anInteger! !

!PocketCubeScrambleOperation methodsFor: 'as yet unclassified' stamp: 'hjo 2/7/2011 13:30'!
operations

	^Array streamContents: [:stream|
		stream nextPutAll: (self preoperations).
		stream nextPutAll: (self twistOperations).
		stream nextPutAll: (self postoperations).
		].! !

!PocketCubeScrambleOperation methodsFor: 'as yet unclassified' stamp: 'hjo 2/7/2011 13:31'!
sameTypeWith: aScramble

	^self isKindOf: aScramble class! !

!PocketCubeScrambleOperation methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 12:25'!
setRandomAmount

	self setRandomAmount: Collection randomForPicking! !

!PocketCubeScrambleOperation methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 12:24'!
setRandomAmount: aRandomStream

	|array|
	array := #(1 2 -1).
	amount := Collection mutexForPicking critical: [ array atRandom: aRandomStream ]! !

!PocketCubeScrambleOperation methodsFor: 'as yet unclassified' stamp: 'hjo 2/7/2011 13:14'!
twistOperations

	^Array streamContents: [:stream|
		self amount abs timesRepeat: [
				stream nextPut: self twistSelector].]! !

PocketCubeScrambleOperation subclass: #ScrambleX
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube'!

!ScrambleX methodsFor: 'as yet unclassified' stamp: 'hjo 2/7/2011 13:09'!
postoperations

	^#()! !

!ScrambleX methodsFor: 'as yet unclassified' stamp: 'hjo 2/7/2011 13:05'!
preoperations

	^#()! !

!ScrambleX methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 09:59'!
twistSelector

	^(amount > 0 ifTrue: [PoTwistLowerLayerLeft] ifFalse: [PoTwistLowerLayerRight]) singleton.! !

PocketCubeScrambleOperation subclass: #ScrambleY
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube'!

!ScrambleY methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 10:00'!
postoperations

	^{PoRotateClockwise singleton}! !

!ScrambleY methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 10:00'!
preoperations

	^{PoRotateCounterClockwise singleton}! !

!ScrambleY methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 10:00'!
twistSelector
	^(amount > 0 ifTrue: [PoTwistUpperLayerRight] ifFalse: [PoTwistUpperLayerLeft]) singleton.! !

PocketCubeScrambleOperation subclass: #ScrambleZ
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube'!

!ScrambleZ methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 10:00'!
postoperations

	^{PoRotateClockwise. PoRotateLeft} collect: [:each| each  singleton]
! !

!ScrambleZ methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 10:01'!
preoperations

	^({PoRotateRight. PoRotateCounterClockwise} collect: [:each| each singleton])! !

!ScrambleZ methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 10:01'!
twistSelector
	
	^(amount > 0 ifTrue: [PoTwistLowerLayerLeft] ifFalse: [PoTwistLowerLayerRight]) singleton! !

Object subclass: #PocketCubeScrambler
	instanceVariableNames: 'sequence'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube'!

!PocketCubeScrambler methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 11:01'!
applyTo: aPocketCube

	self translateSequenceToNetStyle do: [:each| each applyTo: aPocketCube ]! !

!PocketCubeScrambler methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 12:35'!
firstInverseOperation
	^ self translateSequenceToNetStyle last inverseOperation! !

!PocketCubeScrambler methodsFor: 'as yet unclassified' stamp: 'hjo 2/7/2011 12:08'!
netStyleScrambleSequenceOf: anInteger

	self scrambleSequenceOf: anInteger.
	^self translateSequenceToNetStyle! !

!PocketCubeScrambler methodsFor: 'as yet unclassified' stamp: 'hjo 7/6/2010 09:01'!
printOnTranscriptScrambleSequenceOf: anInteger

	Transcript show: (self scrambleSequenceOf: anInteger); cr; flush! !

!PocketCubeScrambler methodsFor: 'as yet unclassified' stamp: 'hjo 2/7/2011 18:45'!
printScramblingCode

	^String streamContents: [:stream| 
		self translateSequenceToNetStyle do: [:selector| 
			stream tab; tab; nextPutAll: selector; nextPut: $;; cr 
			]
		]! !

!PocketCubeScrambler methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 11:05'!
rollbackTo: aPocketCube

	self translateSequenceToNetStyle reverseDo: 
		[:each| each inverseOperation applyTo: aPocketCube]! !

!PocketCubeScrambler methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 10:54'!
scrambleSequenceOf: anInteger

	^self scrambleSequenceOf: anInteger randomStream: Collection randomForPicking! !

!PocketCubeScrambler methodsFor: 'as yet unclassified' stamp: 'hjo 3/25/2011 12:24'!
scrambleSequenceOf: anInteger randomStream: aRandom
	| d s randomScramble |
	d := PocketCubeScrambleOperation subclasses.
	s := OrderedCollection new.
	anInteger
		timesRepeat: [ 
			[ 
			randomScramble := (Collection mutexForPicking critical: [ d atRandom: aRandom ]) new.
			randomScramble setRandomAmount: aRandom.
			s isEmpty not and: [ s last sameTypeWith: randomScramble ] ] whileTrue.
			s add: randomScramble ].
	^ sequence := s asArray! !

!PocketCubeScrambler methodsFor: 'as yet unclassified' stamp: 'hjo 2/7/2011 13:23'!
translateSequenceToNetStyle

	| code |
	^Array streamContents: [:stream|
		sequence do: [:each| stream nextPutAll: each operations]
		]
	
! !

Object subclass: #PocketCubeSide
	instanceVariableNames: 'upperLeftPiece upperRightPiece lowerRightPiece lowerLeftPiece'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube'!

!PocketCubeSide classSide methodsFor: 'as yet unclassified' stamp: 'hjo 7/26/2010 00:47'!
ofAllSamePieces: aColor 

	^self upperLeftPiece: aColor upperRightPiece: aColor lowerLeftPiece: aColor lowerRightPiece: aColor ! !

!PocketCubeSide classSide methodsFor: 'as yet unclassified' stamp: 'hjo 7/17/2010 19:45'!
upperLeftPiece: aSmallInteger upperRightPiece: aSmallInteger2 lowerLeftPiece: aSmallInteger3 lowerRightPiece: aSmallInteger4 

	| instance |
	instance := self new.
	instance upperLeftPiece: aSmallInteger.
	instance upperRightPiece: aSmallInteger2.
	instance lowerLeftPiece: aSmallInteger3.
	instance lowerRightPiece: aSmallInteger4.
	^instance! !

!PocketCubeSide methodsFor: 'comparing' stamp: 'hjo 7/18/2010 01:29'!
= other

	^self equalityComparison of: self and: other! !

!PocketCubeSide methodsFor: 'as yet unclassified' stamp: 'hjo 7/17/2010 20:03'!
copyInLowerLayer: anArray 

	lowerLeftPiece := anArray first.
	lowerRightPiece := anArray second! !

!PocketCubeSide methodsFor: 'as yet unclassified' stamp: 'hjo 7/17/2010 19:58'!
copyInUpperLayer: anArray 

	upperLeftPiece := anArray first.
	upperRightPiece := anArray second! !

!PocketCubeSide methodsFor: 'as yet unclassified' stamp: 'hjo 7/17/2010 19:55'!
copyOutLowerLayer

	^Array with: lowerLeftPiece with: lowerRightPiece! !

!PocketCubeSide methodsFor: 'as yet unclassified' stamp: 'hjo 7/17/2010 19:53'!
copyOutUpperLayer

	^Array with: upperLeftPiece with: upperRightPiece! !

!PocketCubeSide methodsFor: 'comparing' stamp: 'hjo 7/18/2010 01:28'!
equalityComparison

	^EqualityComparison new selectors: #(upperLeftPiece upperRightPiece lowerLeftPiece lowerRightPiece)! !

!PocketCubeSide methodsFor: 'comparing' stamp: 'hjo 7/18/2010 01:33'!
hash

	^self equalityComparison hashOf: self! !

!PocketCubeSide methodsFor: 'testing' stamp: 'hjo 1/31/2011 19:13'!
isOneColor

	^(Set with: upperLeftPiece with: upperRightPiece with: lowerLeftPiece with: lowerRightPiece) size = 1! !

!PocketCubeSide methodsFor: 'as yet unclassified' stamp: 'hjo 7/17/2010 19:44'!
lowerLeftPiece

	^lowerLeftPiece! !

!PocketCubeSide methodsFor: 'as yet unclassified' stamp: 'hjo 7/17/2010 19:44'!
lowerLeftPiece: aSmallInteger 
	lowerLeftPiece := aSmallInteger ! !

!PocketCubeSide methodsFor: 'as yet unclassified' stamp: 'hjo 7/17/2010 19:32'!
lowerRightPiece

	^lowerRightPiece! !

!PocketCubeSide methodsFor: 'as yet unclassified' stamp: 'hjo 7/17/2010 19:30'!
lowerRightPiece: aSmallInteger 

	lowerRightPiece := aSmallInteger ! !

!PocketCubeSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 21:51'!
pieceSet

	^Set with: upperLeftPiece with: upperRightPiece with: lowerLeftPiece with: lowerRightPiece.! !

!PocketCubeSide methodsFor: 'as yet unclassified' stamp: 'hjo 7/18/2010 07:56'!
printPieces

	^String streamContents: [:stream|
		stream print: upperLeftPiece.
		stream space.
		stream print: upperRightPiece.
		stream cr.
		stream print: lowerLeftPiece.
		stream space.
		stream print: lowerRightPiece]! !

!PocketCubeSide methodsFor: 'as yet unclassified' stamp: 'hjo 7/17/2010 19:43'!
rotateClockwise

	|temp|
	temp := lowerLeftPiece.
	lowerLeftPiece := lowerRightPiece.
	lowerRightPiece := upperRightPiece.
	upperRightPiece := upperLeftPiece.
	upperLeftPiece := temp.
	! !

!PocketCubeSide methodsFor: 'as yet unclassified' stamp: 'hjo 7/17/2010 19:48'!
rotateCounterClockwise

	| temp |
	temp := upperLeftPiece.
	upperLeftPiece := upperRightPiece.
	upperRightPiece := lowerRightPiece.
	lowerRightPiece := lowerLeftPiece.
	lowerLeftPiece := temp! !

!PocketCubeSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/7/2011 10:20'!
rotatedCounterClockwisely

	^self species 
		upperLeftPiece: upperRightPiece  upperRightPiece: lowerRightPiece
		lowerLeftPiece: upperLeftPiece lowerRightPiece: lowerLeftPiece! !

!PocketCubeSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 13:32'!
sidePattern

	#(UnifiedSidePattern BottomBarSidePattern #VerticallyParallelSidePattern LeftBarSidePattern RightBarSidePattern TopBarSidePattern BottomLeftLSidePattern HorizontallyParallelSidePattern BottomRightLSidePattern TopLeftLSidePattern TopRightLSidePattern FourColorSidePattern) do: [:each| | pattern |
			pattern := (Smalltalk at: each) for: self.
			pattern ifNotNil: [^pattern]].
	^nil! !

!PocketCubeSide methodsFor: 'as yet unclassified' stamp: 'hjo 7/17/2010 19:31'!
upperLeftPiece

	^upperLeftPiece! !

!PocketCubeSide methodsFor: 'as yet unclassified' stamp: 'hjo 7/17/2010 19:29'!
upperLeftPiece: aSmallInteger 

	upperLeftPiece := aSmallInteger ! !

!PocketCubeSide methodsFor: 'as yet unclassified' stamp: 'hjo 7/17/2010 19:31'!
upperRightPiece

	^upperRightPiece! !

!PocketCubeSide methodsFor: 'as yet unclassified' stamp: 'hjo 7/17/2010 19:29'!
upperRightPiece: aSmallInteger 

	upperRightPiece := aSmallInteger ! !

Object subclass: #PocketCubeSideController
	instanceVariableNames: 'pocketCube sideAccessor upperLeftPieceMorph upperRightPieceMorph lowerRightPieceMorph lowerLeftPieceMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-Morph'!

!PocketCubeSideController classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 19:44'!
openMorphOn: aPocketCube sideAccessor: aByteSymbol atOrigin: aPoint gameMorph: gameMorph 

	| instance |
	instance := self new.
	instance pocketCube: aPocketCube.
	instance sideAccessor: aByteSymbol.
	instance openMorphAtOrigin: aPoint gameMorph: gameMorph.
	instance refreshMorph.
	^instance! !

!PocketCubeSideController methodsFor: 'as yet unclassified' stamp: 'hjo 7/26/2010 01:35'!
openBorderedRectangleMorph

	| morph | 
	morph := RectangleMorph new.
	morph borderWidth: 2.
	morph extent: 40@40.
	^morph! !

!PocketCubeSideController methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 12:52'!
openMorphAtOrigin: aPoint gameMorph: gameMorph 

	upperLeftPieceMorph := self openMorphAtOrigin: aPoint gameMorph: gameMorph pieceOffset: 0@0 color: self pocketCubeSide upperLeftPiece.
	upperRightPieceMorph := self openMorphAtOrigin: aPoint gameMorph: gameMorph pieceOffset: 42@0 color: self pocketCubeSide upperRightPiece.
	lowerRightPieceMorph := self openMorphAtOrigin: aPoint gameMorph: gameMorph pieceOffset: 42@42 color: self pocketCubeSide lowerRightPiece.
	lowerLeftPieceMorph := self openMorphAtOrigin: aPoint gameMorph: gameMorph pieceOffset: 0@42 color: self pocketCubeSide lowerLeftPiece.! !

!PocketCubeSideController methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 12:53'!
openMorphAtOrigin: aPoint gameMorph: gameMorph pieceOffset: offset color: aColor
	
	| pieceMorph |
	pieceMorph := self openBorderedRectangleMorph.
	gameMorph addMorph: pieceMorph.
	pieceMorph position: gameMorph position + offset + aPoint.
	pieceMorph color: aColor.
	^pieceMorph! !

!PocketCubeSideController methodsFor: 'as yet unclassified' stamp: 'hjo 7/26/2010 02:11'!
pocketCube: aPocketCube 

	pocketCube := aPocketCube! !

!PocketCubeSideController methodsFor: 'as yet unclassified' stamp: 'hjo 7/26/2010 02:08'!
pocketCubeSide

	^pocketCube perform: sideAccessor! !

!PocketCubeSideController methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 19:25'!
refreshMorph

	upperLeftPieceMorph color: self pocketCubeSide upperLeftPiece.
	upperRightPieceMorph color: self pocketCubeSide upperRightPiece.
	lowerLeftPieceMorph color: self pocketCubeSide lowerLeftPiece.
	lowerRightPieceMorph color: self pocketCubeSide lowerRightPiece.! !

!PocketCubeSideController methodsFor: 'as yet unclassified' stamp: 'hjo 7/26/2010 02:11'!
sideAccessor: aByteSymbol 

	sideAccessor := aByteSymbol! !

PocketCubeSideController subclass: #PocketCubeVirtualSideController
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-Morph'!

!PocketCubeVirtualSideController methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 19:28'!
alpha

	^0.3! !

!PocketCubeVirtualSideController methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 19:34'!
noBoarder

	upperLeftPieceMorph borderWidth: 0.
	upperRightPieceMorph borderWidth: 0.
	lowerLeftPieceMorph borderWidth: 0.
	lowerRightPieceMorph borderWidth: 0.! !

!PocketCubeVirtualSideController methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 19:28'!
refreshAlpha.

	upperLeftPieceMorph color: (upperLeftPieceMorph color alpha: self alpha).
	lowerLeftPieceMorph color: (lowerLeftPieceMorph color alpha: self alpha).
	upperRightPieceMorph color: (upperRightPieceMorph color alpha: self alpha).
	lowerRightPieceMorph color: (lowerRightPieceMorph color alpha: self alpha).! !

!PocketCubeVirtualSideController methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 19:33'!
refreshMorph

	super refreshMorph.
	self refreshAlpha.
	self noBoarder.! !

Object subclass: #PocketCubeWorkspace
	instanceVariableNames: 'c s scr'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube'!

!PocketCubeWorkspace methodsFor: 'as yet unclassified' stamp: 'hjo 2/21/2011 15:25'!
deleteAllMorphs
	
	PocketCubeGameMorph allInstances do: [:each| each delete ].
! !

!PocketCubeWorkspace methodsFor: 'as yet unclassified' stamp: 'hjo 2/21/2011 15:48'!
generatePatternClass

	(PocketCubePatternGenerator on: c model withScrambler:scr) generatePatternClass.! !

!PocketCubeWorkspace methodsFor: 'as yet unclassified' stamp: 'hjo 2/21/2011 15:26'!
oneStepBetter

	s oneStepBetter.! !

!PocketCubeWorkspace methodsFor: 'as yet unclassified' stamp: 'hjo 2/21/2011 15:31'!
printPatternMatchingCode

	^(PocketCubePatternGenerator on: c model withScrambler:scr) patternMatchingCode.! !

!PocketCubeWorkspace methodsFor: 'as yet unclassified' stamp: 'hjo 2/21/2011 15:30'!
printScramblingCode

	^scr printScramblingCode! !

!PocketCubeWorkspace methodsFor: 'as yet unclassified' stamp: 'hjo 2/21/2011 15:25'!
reset

	self deleteAllMorphs.
	c := PocketCubeController new.
	s := c solve.
	c refreshMorph.
! !

!PocketCubeWorkspace methodsFor: 'as yet unclassified' stamp: 'hjo 2/21/2011 15:29'!
scramble

	scr applyTo: c! !

!PocketCubeWorkspace methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 19:39'!
setScrambler

	self setScramblerOfOrder: 3! !

!PocketCubeWorkspace methodsFor: 'as yet unclassified' stamp: 'hjo 3/24/2011 19:38'!
setScramblerOfOrder: anInteger

	scr := PocketCubeScrambler new.
	scr netStyleScrambleSequenceOf: anInteger.
! !

!PocketCubeWorkspace methodsFor: 'as yet unclassified' stamp: 'hjo 2/21/2011 15:29'!
unscramble

	scr rollbackTo: c! !

Object subclass: #SidePattern
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-SidePatterns'!

SidePattern subclass: #BarSidePattern
	instanceVariableNames: 'barColor colorOfLeftToTheBar colorOfRightToTheBar'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-SidePatterns'!

!BarSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 23:11'!
confirmedFor: aPocketCubeSide

	| instance |
	instance := self new.
	instance barColor: (aPocketCubeSide perform: self barColorGetter).
	instance colorOfLeftToTheBar: (aPocketCubeSide perform: self colorOfLeftToTheBarGetter).
	instance colorOfRightToTheBar: (aPocketCubeSide perform: self colorOfRightToTheBarGetter).
	^instance! !

!BarSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 12:23'!
numberOfColor

	^3! !

!BarSidePattern methodsFor: 'accessing' stamp: 'hjo 2/3/2011 21:31'!
barColor
	^ barColor! !

!BarSidePattern methodsFor: 'accessing' stamp: 'hjo 2/3/2011 21:31'!
barColor: anObject
	barColor := anObject! !

!BarSidePattern methodsFor: 'accessing' stamp: 'hjo 2/3/2011 21:32'!
colorOfLeftToTheBar
	^ colorOfLeftToTheBar! !

!BarSidePattern methodsFor: 'accessing' stamp: 'hjo 2/3/2011 21:32'!
colorOfLeftToTheBar: anObject
	colorOfLeftToTheBar := anObject! !

!BarSidePattern methodsFor: 'accessing' stamp: 'hjo 2/3/2011 21:32'!
colorOfRightToTheBar
	^ colorOfRightToTheBar! !

!BarSidePattern methodsFor: 'accessing' stamp: 'hjo 2/3/2011 21:32'!
colorOfRightToTheBar: anObject
	colorOfRightToTheBar := anObject! !

BarSidePattern subclass: #BottomBarSidePattern
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-SidePatterns'!

!BottomBarSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 23:13'!
barColorGetter

	^#lowerLeftPiece! !

!BottomBarSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 22:48'!
checkBarPieceMatch: aPocketCubeSide

	^aPocketCubeSide lowerLeftPiece = aPocketCubeSide lowerRightPiece! !

!BottomBarSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 23:14'!
colorOfLeftToTheBarGetter

	^#upperLeftPiece! !

!BottomBarSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 23:14'!
colorOfRightToTheBarGetter

	^#upperRightPiece! !

BarSidePattern subclass: #LeftBarSidePattern
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-SidePatterns'!

!LeftBarSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 23:15'!
barColorGetter

	^#upperLeftPiece! !

!LeftBarSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 22:49'!
checkBarPieceMatch: aPocketCubeSide

	^aPocketCubeSide lowerLeftPiece = aPocketCubeSide upperLeftPiece! !

!LeftBarSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 23:15'!
colorOfLeftToTheBarGetter

	^#upperRightPiece! !

!LeftBarSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 23:22'!
colorOfRightToTheBarGetter

	^#lowerRightPiece! !

!LeftBarSidePattern methodsFor: 'as yet unclassified' stamp: 'hjo 2/7/2011 19:39'!
isLeftBar

	^true! !

BarSidePattern subclass: #RightBarSidePattern
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-SidePatterns'!

!RightBarSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 23:22'!
barColorGetter

	^#upperRightPiece! !

!RightBarSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 22:50'!
checkBarPieceMatch: aPocketCubeSide

	^aPocketCubeSide lowerRightPiece = aPocketCubeSide upperRightPiece! !

!RightBarSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 23:23'!
colorOfLeftToTheBarGetter

	^#lowerLeftPiece! !

!RightBarSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 23:23'!
colorOfRightToTheBarGetter

	^#upperLeftPiece! !

!RightBarSidePattern methodsFor: 'as yet unclassified' stamp: 'hjo 2/7/2011 19:39'!
isRightBar

	^true! !

BarSidePattern subclass: #TopBarSidePattern
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-SidePatterns'!

!TopBarSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 23:24'!
barColorGetter

	^#upperRightPiece! !

!TopBarSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 22:50'!
checkBarPieceMatch: aPocketCubeSide

	^aPocketCubeSide upperLeftPiece = aPocketCubeSide upperRightPiece! !

!TopBarSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 23:24'!
colorOfLeftToTheBarGetter

	^#lowerRightPiece! !

!TopBarSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 23:24'!
colorOfRightToTheBarGetter

	^#lowerLeftPiece! !

SidePattern subclass: #FourColorSidePattern
	instanceVariableNames: 'side'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-SidePatterns'!

!FourColorSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 13:33'!
checkBarPieceMatch: aPocketCubeSide 

	^true! !

!FourColorSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 13:34'!
confirmedFor: aPocketCubeSide 

	| instance |
	instance := self new.
	instance side: aPocketCubeSide copy.
	^instance.! !

!FourColorSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 13:32'!
numberOfColor

	^4! !

!FourColorSidePattern methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 13:35'!
side

	^side! !

!FourColorSidePattern methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 13:34'!
side: aPocketCubeSide 

	side := aPocketCubeSide! !

SidePattern subclass: #LSidePattern
	instanceVariableNames: 'colorOfL colorOfDot'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-SidePatterns'!

LSidePattern subclass: #BottomLeftLSidePattern
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-SidePatterns'!

!BottomLeftLSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 00:39'!
checkBarPieceMatch: aPocketCubeSide 

	^(Set with: aPocketCubeSide upperLeftPiece with: aPocketCubeSide lowerLeftPiece with: aPocketCubeSide lowerRightPiece) size = 1! !

!BottomLeftLSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 12:42'!
colorOfDotGetter

	^#upperRightPiece! !

!BottomLeftLSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 12:42'!
colorOfLGetter

	^#lowerLeftPiece.! !

LSidePattern subclass: #BottomRightLSidePattern
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-SidePatterns'!

!BottomRightLSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 00:31'!
checkBarPieceMatch: aPocketCubeSide 

	^(Set with: aPocketCubeSide upperRightPiece with: aPocketCubeSide lowerLeftPiece with: aPocketCubeSide lowerRightPiece) size = 1! !

!BottomRightLSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 12:44'!
colorOfDotGetter

	^#upperLeftPiece.! !

!BottomRightLSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 12:43'!
colorOfLGetter

	^#lowerRightPiece.! !

!LSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 12:41'!
confirmedFor: aPocketCubeSide 

	| instance |
	instance := self new.
	instance colorOfL: (aPocketCubeSide perform: self colorOfLGetter).
	instance colorOfDot: (aPocketCubeSide perform: self colorOfDotGetter).
	^instance! !

!LSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 12:27'!
numberOfColor

	^2! !

!LSidePattern methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 00:33'!
colorOfDot

	^colorOfDot! !

!LSidePattern methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 00:33'!
colorOfDot: aColor 

	colorOfDot := aColor ! !

!LSidePattern methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 00:33'!
colorOfL

	^colorOfL! !

!LSidePattern methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 00:34'!
colorOfL: aColor 

	colorOfL := aColor! !

LSidePattern subclass: #TopLeftLSidePattern
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-SidePatterns'!

!TopLeftLSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 12:19'!
checkBarPieceMatch: aPocketCubeSide 

	^(Set with: aPocketCubeSide upperLeftPiece with: aPocketCubeSide upperRightPiece with: aPocketCubeSide lowerLeftPiece) size = 1! !

!TopLeftLSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 12:45'!
colorOfDotGetter

	^#lowerRightPiece.! !

!TopLeftLSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 12:44'!
colorOfLGetter

	^#upperLeftPiece.! !

LSidePattern subclass: #TopRightLSidePattern
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-SidePatterns'!

!TopRightLSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 12:15'!
checkBarPieceMatch: aPocketCubeSide 

	^(Set with: aPocketCubeSide upperLeftPiece
	with: aPocketCubeSide upperRightPiece
	with: aPocketCubeSide lowerRightPiece) size = 1! !

!TopRightLSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 12:45'!
colorOfDotGetter

	^#lowerLeftPiece.! !

!TopRightLSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 12:45'!
colorOfLGetter

	^#upperRightPiece.! !

SidePattern subclass: #ParallelSidePattern
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-SidePatterns'!

ParallelSidePattern subclass: #HorizontallyParallelSidePattern
	instanceVariableNames: 'colorOfTopBar colorOfBottomBar'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-SidePatterns'!

!HorizontallyParallelSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/4/2011 13:19'!
checkBarPieceMatch: aPocketCubeSide

	^(aPocketCubeSide lowerLeftPiece = aPocketCubeSide lowerRightPiece)
		and: [aPocketCubeSide upperLeftPiece = aPocketCubeSide upperRightPiece]! !

!HorizontallyParallelSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/4/2011 13:35'!
confirmedFor: aPocketCubeSide

	| instance |
	instance := self new.
	instance colorOfTopBar: aPocketCubeSide upperLeftPiece.
	instance colorOfBottomBar: aPocketCubeSide lowerLeftPiece.
	^instance! !

!HorizontallyParallelSidePattern methodsFor: 'accessing' stamp: 'hjo 2/4/2011 13:37'!
colorOfBottomBar
	^ colorOfBottomBar! !

!HorizontallyParallelSidePattern methodsFor: 'accessing' stamp: 'hjo 2/4/2011 13:37'!
colorOfBottomBar: anObject
	colorOfBottomBar := anObject! !

!HorizontallyParallelSidePattern methodsFor: 'accessing' stamp: 'hjo 2/4/2011 13:13'!
colorOfTopBar
	^ colorOfTopBar! !

!HorizontallyParallelSidePattern methodsFor: 'accessing' stamp: 'hjo 2/4/2011 13:13'!
colorOfTopBar: anObject
	colorOfTopBar := anObject! !

!ParallelSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 12:26'!
numberOfColor

	^2! !

!ParallelSidePattern methodsFor: 'as yet unclassified' stamp: 'hjo 2/4/2011 13:49'!
isParallelPattern

	^true! !

ParallelSidePattern subclass: #VerticallyParallelSidePattern
	instanceVariableNames: 'colorOfLeftBar colorOfRightBar'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-SidePatterns'!

!VerticallyParallelSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/4/2011 13:52'!
checkBarPieceMatch: aPocketCubeSide

	^(aPocketCubeSide lowerLeftPiece = aPocketCubeSide upperLeftPiece)
		and: [aPocketCubeSide lowerRightPiece = aPocketCubeSide upperRightPiece]! !

!VerticallyParallelSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/4/2011 13:53'!
confirmedFor: aPocketCubeSide

	| instance |
	instance := self new.
	instance colorOfLeftBar: aPocketCubeSide upperLeftPiece.
	instance colorOfRightBar: aPocketCubeSide upperRightPiece.
	^instance! !

!VerticallyParallelSidePattern methodsFor: 'as yet unclassified' stamp: 'hjo 2/4/2011 13:44'!
colorOfLeftBar

	^colorOfLeftBar! !

!VerticallyParallelSidePattern methodsFor: 'accessing' stamp: 'hjo 2/3/2011 21:52'!
colorOfLeftBar: aColor 

	colorOfLeftBar := aColor ! !

!VerticallyParallelSidePattern methodsFor: 'as yet unclassified' stamp: 'hjo 2/4/2011 13:44'!
colorOfRightBar

	^colorOfRightBar! !

!VerticallyParallelSidePattern methodsFor: 'accessing' stamp: 'hjo 2/3/2011 21:52'!
colorOfRightBar: aColor 

	colorOfRightBar := aColor ! !

!SidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 00:50'!
checkMatch: aPocketCubeSide 

	^(self checkPieceSetSizeMatch: aPocketCubeSide) and: [self checkBarPieceMatch: aPocketCubeSide]! !

!SidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 12:24'!
checkPieceSetSizeMatch: aPocketCubeSide 

	^aPocketCubeSide pieceSet size = self numberOfColor! !

!SidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 12:34'!
for: aPocketCubeSide 

	(self checkMatch: aPocketCubeSide) ifFalse: [^nil].
	^self confirmedFor: aPocketCubeSide.! !

!SidePattern methodsFor: 'as yet unclassified' stamp: 'hjo 2/7/2011 19:39'!
isLeftBar

	^false! !

!SidePattern methodsFor: 'as yet unclassified' stamp: 'hjo 2/3/2011 21:33'!
isParallelPattern

	^false! !

!SidePattern methodsFor: 'as yet unclassified' stamp: 'hjo 2/7/2011 19:39'!
isRightBar

	^false! !

SidePattern subclass: #UnifiedSidePattern
	instanceVariableNames: 'color'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PocketCube-SidePatterns'!

!UnifiedSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 00:51'!
checkBarPieceMatch: aPocketCubeSide 

	^true! !

!UnifiedSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 00:53'!
confirmedFor: aPocketCubeSide 

	|instance|
	instance := self new.
	instance color: aPocketCubeSide upperLeftPiece.
	^instance ! !

!UnifiedSidePattern classSide methodsFor: 'as yet unclassified' stamp: 'hjo 2/5/2011 12:26'!
numberOfColor

	^1! !

!UnifiedSidePattern methodsFor: 'accessing' stamp: 'hjo 2/3/2011 21:16'!
color
	^ color! !

!UnifiedSidePattern methodsFor: 'accessing' stamp: 'hjo 2/3/2011 21:16'!
color: aColor
	color := aColor! !
PocketCubeRecognizer initialize!
